@@ -21,7 +21,12 @@
 #include "atkmarshal.h"
 
 enum {
-  MODEL_CHANGED,
+  ROW_INSERTED,
+  ROW_DELETED,
+  COLUMN_INSERTED,
+  COLUMN_DELETED,
+  ROW_REORDERED,
+  COLUMN_REORDERED,
   LAST_SIGNAL
 };
 
@@ -32,7 +37,15 @@ struct _AtkTableIfaceClass
 
 typedef struct _AtkTableIfaceClass AtkTableIfaceClass;
 
-static void atk_table_base_init (gpointer *g_class);
+static void  atk_table_base_init (gpointer *g_class);
+
+static gint  atk_table_real_get_index_at        (AtkTable *table,
+                                                 gint     row,
+                                                 gint     column);
+static gint  atk_table_real_get_column_at_index (AtkTable *table,
+                                                 gint     index);
+static gint  atk_table_real_get_row_at_index    (AtkTable *table,
+                                                 gint     index);
 
 static guint atk_table_signals[LAST_SIGNAL] = { 0 };
 
@@ -40,19 +53,19 @@ GType
 atk_table_get_type ()
 {
   static GType type = 0;
-
+  
   if (!type) {
     GTypeInfo tinfo =
     {
       sizeof (AtkTableIface),
       (GBaseInitFunc) atk_table_base_init,
       (GBaseFinalizeFunc) NULL,
-
+      
     };
-
+    
     type = g_type_register_static (G_TYPE_INTERFACE, "AtkTable", &tinfo, 0);
   }
-
+  
   return type;
 }
 
@@ -61,21 +74,65 @@ static void
 atk_table_base_init (gpointer *g_class)
 {
   static gboolean initialized = FALSE;
-
-  if (! initialized)
-  {
-    atk_table_signals[MODEL_CHANGED] =
-      g_signal_newc ("model_changed",
-                     ATK_TYPE_TABLE,
-                     G_SIGNAL_RUN_LAST,
-                     G_STRUCT_OFFSET (AtkTableIface, model_changed),
-                     (GSignalAccumulator) NULL, NULL,
-                     atk_marshal_VOID__INT_INT_INT_INT_INT,
-                     G_TYPE_NONE,
-                     5, G_TYPE_INT, G_TYPE_INT, G_TYPE_INT, G_TYPE_INT, 
-                     G_TYPE_INT);
-    initialized = TRUE;
-  }
+  
+  if (!initialized)
+    {
+      atk_table_signals[ROW_INSERTED] =
+	g_signal_new ("row_inserted",
+		      ATK_TYPE_TABLE,
+		      G_SIGNAL_RUN_LAST,
+		      G_STRUCT_OFFSET (AtkTableIface, row_inserted),
+		      (GSignalAccumulator) NULL, NULL,
+		      atk_marshal_VOID__INT_INT,
+		      G_TYPE_NONE,
+		      2, G_TYPE_INT, G_TYPE_INT);
+      atk_table_signals[COLUMN_INSERTED] =
+	g_signal_new ("column_inserted",
+		      ATK_TYPE_TABLE,
+		      G_SIGNAL_RUN_LAST,
+		      G_STRUCT_OFFSET (AtkTableIface, column_inserted),
+		      (GSignalAccumulator) NULL, NULL,
+		      atk_marshal_VOID__INT_INT,
+		      G_TYPE_NONE,
+		      2, G_TYPE_INT, G_TYPE_INT);
+      atk_table_signals[ROW_DELETED] =
+	g_signal_new ("row_deleted",
+		      ATK_TYPE_TABLE,
+		      G_SIGNAL_RUN_LAST,
+		      G_STRUCT_OFFSET (AtkTableIface, row_deleted),
+		      (GSignalAccumulator) NULL, NULL,
+		      atk_marshal_VOID__INT_INT,
+		      G_TYPE_NONE,
+		      2, G_TYPE_INT, G_TYPE_INT);
+      atk_table_signals[COLUMN_DELETED] =
+	g_signal_new ("column_deleted",
+		      ATK_TYPE_TABLE,
+		      G_SIGNAL_RUN_LAST,
+		      G_STRUCT_OFFSET (AtkTableIface, column_deleted),
+		      (GSignalAccumulator) NULL, NULL,
+		      atk_marshal_VOID__INT_INT,
+		      G_TYPE_NONE,
+		      2, G_TYPE_INT, G_TYPE_INT);
+      atk_table_signals[ROW_REORDERED] =
+	g_signal_new ("row_reordered",
+		      ATK_TYPE_TABLE,
+		      G_SIGNAL_RUN_LAST,
+		      G_STRUCT_OFFSET (AtkTableIface, row_reordered),
+		      (GSignalAccumulator) NULL, NULL,
+		      g_cclosure_marshal_VOID__VOID,
+		      G_TYPE_NONE,
+		      0);
+      atk_table_signals[COLUMN_REORDERED] =
+	g_signal_new ("column_reordered",
+		      ATK_TYPE_TABLE,
+		      G_SIGNAL_RUN_LAST,
+		      G_STRUCT_OFFSET (AtkTableIface, column_reordered),
+		      (GSignalAccumulator) NULL, NULL,
+		      g_cclosure_marshal_VOID__VOID,
+		      G_TYPE_NONE,
+		      0);
+      initialized = TRUE;
+    }
 }
 
 /**
@@ -84,24 +141,23 @@ atk_table_base_init (gpointer *g_class)
  * @row: a #gint representing a row in @table
  * @column: a #gint representing a column in @table
  *
- * Get a reference to the table cell at @row, @column
+ * Get a reference to the table cell at @row, @column.
  *
  * Returns: a AtkObject* representing the referred to accessible
  **/
 AtkObject*
-atk_table_ref_at (AtkTable *obj,
+atk_table_ref_at (AtkTable *table,
                   gint     row,
                   gint     column)
 {
   AtkTableIface *iface;
 
-  g_return_val_if_fail (obj != NULL, NULL);
-  g_return_val_if_fail (ATK_IS_TABLE (obj), NULL);
+  g_return_val_if_fail (ATK_IS_TABLE (table), NULL);
 
-  iface = ATK_TABLE_GET_IFACE (obj);
+  iface = ATK_TABLE_GET_IFACE (table);
 
   if (iface->ref_at)
-    return (iface->ref_at) (obj, row, column);
+    return (iface->ref_at) (table, row, column);
   else
     return NULL;
 }
@@ -123,21 +179,22 @@ atk_table_ref_at (AtkTable *obj,
  * if value does not implement this interface.
  **/
 gint
-atk_table_get_index_at (AtkTable *obj,
+atk_table_get_index_at (AtkTable *table,
                         gint     row,
                         gint     column)
 {
   AtkTableIface *iface;
 
-  g_return_val_if_fail (obj != NULL, 0);
-  g_return_val_if_fail (ATK_IS_TABLE (obj), 0);
+  g_return_val_if_fail (ATK_IS_TABLE (table), 0);
+  g_return_val_if_fail (row >= 0, 0);
+  g_return_val_if_fail (column >= 0, 0);
 
-  iface = ATK_TABLE_GET_IFACE (obj);
+  iface = ATK_TABLE_GET_IFACE (table);
 
   if (iface->get_index_at)
-    return (iface->get_index_at) (obj, row, column);
+    return (iface->get_index_at) (table, row, column);
   else
-    return 0;
+    return atk_table_real_get_index_at (table, row, column);
 }
 
 /**
@@ -156,20 +213,19 @@ atk_table_get_index_at (AtkTable *obj,
  * if value does not implement this interface.
  **/
 gint
-atk_table_get_row_at_index (AtkTable *obj,
+atk_table_get_row_at_index (AtkTable *table,
                             gint     index)
 {
   AtkTableIface *iface;
 
-  g_return_val_if_fail (obj != NULL, 0);
-  g_return_val_if_fail (ATK_IS_TABLE (obj), 0);
+  g_return_val_if_fail (ATK_IS_TABLE (table), 0);
 
-  iface = ATK_TABLE_GET_IFACE (obj);
+  iface = ATK_TABLE_GET_IFACE (table);
 
   if (iface->get_row_at_index)
-    return (iface->get_row_at_index) (obj, index);
+    return (iface->get_row_at_index) (table, index);
   else
-    return 0;
+    return atk_table_real_get_row_at_index (table, index);
 }
 
 /**
@@ -188,20 +244,19 @@ atk_table_get_row_at_index (AtkTable *obj,
  * if value does not implement this interface.
  **/
 gint
-atk_table_get_column_at_index (AtkTable *obj,
+atk_table_get_column_at_index (AtkTable *table,
                                gint     index)
 {
   AtkTableIface *iface;
 
-  g_return_val_if_fail (obj != NULL, 0);
-  g_return_val_if_fail (ATK_IS_TABLE (obj), 0);
+  g_return_val_if_fail (ATK_IS_TABLE (table), 0);
 
-  iface = ATK_TABLE_GET_IFACE (obj);
+  iface = ATK_TABLE_GET_IFACE (table);
 
   if (iface->get_column_at_index)
-    return (iface->get_column_at_index) (obj, index);
+    return (iface->get_column_at_index) (table, index);
   else
-    return 0;
+    return atk_table_real_get_column_at_index (table, index);
 }
 
 /**
@@ -217,18 +272,17 @@ atk_table_get_column_at_index (AtkTable *obj,
  * Returns: a gchar* representing the table caption, or %NULL
  * if value does not implement this interface.
  **/
-gchar*
-atk_table_get_caption (AtkTable *obj)
+G_CONST_RETURN gchar*
+atk_table_get_caption (AtkTable *table)
 {
   AtkTableIface *iface;
 
-  g_return_val_if_fail (obj != NULL, NULL);
-  g_return_val_if_fail (ATK_IS_TABLE (obj), NULL);
+  g_return_val_if_fail (ATK_IS_TABLE (table), NULL);
 
-  iface = ATK_TABLE_GET_IFACE (obj);
+  iface = ATK_TABLE_GET_IFACE (table);
 
   if (iface->get_caption)
-    return (iface->get_caption) (obj);
+    return (iface->get_caption) (table);
   else
     return NULL;
 }
@@ -247,17 +301,16 @@ atk_table_get_caption (AtkTable *obj)
  * if value does not implement this interface.
  **/
 gint
-atk_table_get_n_columns (AtkTable *obj)
+atk_table_get_n_columns (AtkTable *table)
 {
   AtkTableIface *iface;
 
-  g_return_val_if_fail (obj != NULL, 0);
-  g_return_val_if_fail (ATK_IS_TABLE (obj), 0);
+  g_return_val_if_fail (ATK_IS_TABLE (table), 0);
 
-  iface = ATK_TABLE_GET_IFACE (obj);
+  iface = ATK_TABLE_GET_IFACE (table);
 
   if (iface->get_n_columns)
-    return (iface->get_n_columns) (obj);
+    return (iface->get_n_columns) (table);
   else
     return 0;
 }
@@ -276,19 +329,18 @@ atk_table_get_n_columns (AtkTable *obj)
  * Returns: a gchar* representing the column description, or %NULL
  * if value does not implement this interface.
  **/
-gchar*
-atk_table_get_column_description (AtkTable *obj,
+G_CONST_RETURN gchar*
+atk_table_get_column_description (AtkTable *table,
                                   gint     column)
 {
   AtkTableIface *iface;
 
-  g_return_val_if_fail (obj != NULL, NULL);
-  g_return_val_if_fail (ATK_IS_TABLE (obj), NULL);
+  g_return_val_if_fail (ATK_IS_TABLE (table), NULL);
 
-  iface = ATK_TABLE_GET_IFACE (obj);
+  iface = ATK_TABLE_GET_IFACE (table);
 
   if (iface->get_column_description)
-    return (iface->get_column_description) (obj, column);
+    return (iface->get_column_description) (table, column);
   else
     return NULL;
 }
@@ -310,19 +362,18 @@ atk_table_get_column_description (AtkTable *obj,
  * if value does not implement this interface.
  **/
 gint
-atk_table_get_column_extent_at (AtkTable *obj,
+atk_table_get_column_extent_at (AtkTable *table,
                                 gint     row,
                                 gint     column)
 {
   AtkTableIface *iface;
 
-  g_return_val_if_fail (obj != NULL, 0);
-  g_return_val_if_fail (ATK_IS_TABLE (obj), 0);
+  g_return_val_if_fail (ATK_IS_TABLE (table), 0);
 
-  iface = ATK_TABLE_GET_IFACE (obj);
+  iface = ATK_TABLE_GET_IFACE (table);
 
   if (iface->get_column_extent_at)
-    return (iface->get_column_extent_at) (obj, row, column);
+    return (iface->get_column_extent_at) (table, row, column);
   else
     return 0;
 }
@@ -342,17 +393,16 @@ atk_table_get_column_extent_at (AtkTable *obj,
  * %NULL if value does not implement this interface.
  **/
 AtkObject*
-atk_table_get_column_header (AtkTable *obj, gint column)
+atk_table_get_column_header (AtkTable *table, gint column)
 {
   AtkTableIface *iface;
 
-  g_return_val_if_fail (obj != NULL, NULL);
-  g_return_val_if_fail (ATK_IS_TABLE (obj), NULL);
+  g_return_val_if_fail (ATK_IS_TABLE (table), NULL);
 
-  iface = ATK_TABLE_GET_IFACE (obj);
+  iface = ATK_TABLE_GET_IFACE (table);
 
   if (iface->get_column_header)
-    return (iface->get_column_header) (obj, column);
+    return (iface->get_column_header) (table, column);
   else
     return NULL;
 }
@@ -371,17 +421,16 @@ atk_table_get_column_header (AtkTable *obj, gint column)
  * if value does not implement this interface.
  **/
 gint
-atk_table_get_n_rows (AtkTable *obj)
+atk_table_get_n_rows (AtkTable *table)
 {
   AtkTableIface *iface;
 
-  g_return_val_if_fail (obj != NULL, 0);
-  g_return_val_if_fail (ATK_IS_TABLE (obj), 0);
+  g_return_val_if_fail (ATK_IS_TABLE (table), 0);
 
-  iface = ATK_TABLE_GET_IFACE (obj);
+  iface = ATK_TABLE_GET_IFACE (table);
 
   if (iface->get_n_rows)
-    return (iface->get_n_rows) (obj);
+    return (iface->get_n_rows) (table);
   else
     return 0;
 }
@@ -389,7 +438,7 @@ atk_table_get_n_rows (AtkTable *obj)
 /**
  * atk_table_get_row_description:
  * @table: a GObject instance that implements AtkTableIface
- * @r: a #gint representing a row in @table
+ * @row: a #gint representing a row in @table
  *
  * Gets the description text of the specified row in the table
  * Note: callers should not rely on %NULL or on a zero value for
@@ -400,19 +449,18 @@ atk_table_get_n_rows (AtkTable *obj)
  * Returns: a gchar* representing the row description, or %NULL
  * if value does not implement this interface.
  **/
-gchar*
-atk_table_get_row_description (AtkTable *obj,
+G_CONST_RETURN gchar*
+atk_table_get_row_description (AtkTable *table,
                                gint      row)
 {
   AtkTableIface *iface;
 
-  g_return_val_if_fail (obj != NULL, NULL);
-  g_return_val_if_fail (ATK_IS_TABLE (obj), NULL);
+  g_return_val_if_fail (ATK_IS_TABLE (table), NULL);
 
-  iface = ATK_TABLE_GET_IFACE (obj);
+  iface = ATK_TABLE_GET_IFACE (table);
 
   if (iface->get_row_description)
-    return (iface->get_row_description) (obj, row);
+    return (iface->get_row_description) (table, row);
   else
     return NULL;
 }
@@ -434,19 +482,18 @@ atk_table_get_row_description (AtkTable *obj,
  * if value does not implement this interface.
  **/
 gint
-atk_table_get_row_extent_at (AtkTable *obj,
+atk_table_get_row_extent_at (AtkTable *table,
                              gint     row,
                              gint     column)
 {
   AtkTableIface *iface;
 
-  g_return_val_if_fail (obj != NULL, 0);
-  g_return_val_if_fail (ATK_IS_TABLE (obj), 0);
+  g_return_val_if_fail (ATK_IS_TABLE (table), 0);
 
-  iface = ATK_TABLE_GET_IFACE (obj);
+  iface = ATK_TABLE_GET_IFACE (table);
 
   if (iface->get_row_extent_at)
-    return (iface->get_row_extent_at) (obj, row, column);
+    return (iface->get_row_extent_at) (table, row, column);
   else
     return 0;
 }
@@ -466,17 +513,16 @@ atk_table_get_row_extent_at (AtkTable *obj,
  * %NULL if value does not implement this interface.
  **/
 AtkObject*
-atk_table_get_row_header (AtkTable *obj, gint row)
+atk_table_get_row_header (AtkTable *table, gint row)
 {
   AtkTableIface *iface;
 
-  g_return_val_if_fail (obj != NULL, NULL);
-  g_return_val_if_fail (ATK_IS_TABLE (obj), NULL);
+  g_return_val_if_fail (ATK_IS_TABLE (table), NULL);
 
-  iface = ATK_TABLE_GET_IFACE (obj);
+  iface = ATK_TABLE_GET_IFACE (table);
 
   if (iface->get_row_header)
-    return (iface->get_row_header) (obj, row);
+    return (iface->get_row_header) (table, row);
   else
     return NULL;
 }
@@ -491,21 +537,20 @@ atk_table_get_row_header (AtkTable *obj, gint row)
  * use type checking/interface checking macros or the
  * atk_get_accessible_table() convenience method.
  *
- * Returns: a AtkObject* representing a sumary description of the table,
- * or %NULL if value does not implement this interface.
+ * Returns: a AtkObject* representing a summary description of the table,
+ * or zero if value does not implement this interface.
  **/
 AtkObject*
-atk_table_get_summary (AtkTable *obj)
+atk_table_get_summary (AtkTable *table)
 {
   AtkTableIface *iface;
 
-  g_return_val_if_fail (obj != NULL, NULL);
-  g_return_val_if_fail (ATK_IS_TABLE (obj), NULL);
+  g_return_val_if_fail (ATK_IS_TABLE (table), NULL);
 
-  iface = ATK_TABLE_GET_IFACE (obj);
+  iface = ATK_TABLE_GET_IFACE (table);
 
   if (iface->get_summary)
-    return (iface->get_summary) (obj);
+    return (iface->get_summary) (table);
   else
     return NULL;
 }
@@ -513,60 +558,61 @@ atk_table_get_summary (AtkTable *obj)
 /**
  * atk_table_get_selected_rows:
  * @table: a GObject instance that implements AtkTableIface
+ * @selected: a #gint** that is to contain the selected row numbers
  *
- * Gets the selected rows of the table.
+ * Gets the selected rows of the table by initializing **selected with 
+ * the selected row numbers. This array should be freed by the caller.
  * Note: callers should not rely on %NULL or on a zero value for
  * indication of whether AtkSelectionIface is implemented, they should
  * use type checking/interface checking macros or the
  * atk_get_accessible_table() convenience method.
  *
- * Returns: a gint* representing the selected rows,
- * or %NULL if value does not implement this interface.
+ * Returns: a gint representing the number of selected rows,
+ * or zero if value does not implement this interface.
  **/
-gint*
-atk_table_get_selected_rows (AtkTable *obj)
+gint
+atk_table_get_selected_rows (AtkTable *table, gint **selected)
 {
   AtkTableIface *iface;
 
-  g_return_val_if_fail (obj != NULL, NULL);
-  g_return_val_if_fail (ATK_IS_TABLE (obj), NULL);
+  g_return_val_if_fail (ATK_IS_TABLE (table), 0);
 
-  iface = ATK_TABLE_GET_IFACE (obj);
+  iface = ATK_TABLE_GET_IFACE (table);
 
   if (iface->get_selected_rows)
-    return (iface->get_selected_rows) (obj);
+    return (iface->get_selected_rows) (table, selected);
   else
-    return NULL;
+    return 0;
 }
 
 /**
  * atk_table_get_selected_columns:
  * @table: a GObject instance that implements AtkTableIface
+ * @selected: a #gint** that is to contain the selected columns numbers
  *
- * Gets the selected columns of the table.
+ * Gets the selected columns of the table by initializing **selected with 
+ * the selected column numbers. This array should be freed by the caller.
  * Note: callers should not rely on %NULL or on a zero value for
  * indication of whether AtkSelectionIface is implemented, they should
  * use type checking/interface checking macros or the
  * atk_get_accessible_table() convenience method.
  *
- * Returns: a gint* representing the selected columns,
- * or %NULL if value does not implement this interface.
- *
+ * Returns: a gint representing the number of selected columns,
+ * or %0 if value does not implement this interface.
  **/
-gint*
-atk_table_get_selected_columns (AtkTable *obj)
+gint 
+atk_table_get_selected_columns (AtkTable *table, gint **selected)
 {
   AtkTableIface *iface;
 
-  g_return_val_if_fail (obj != NULL, NULL);
-  g_return_val_if_fail (ATK_IS_TABLE (obj), NULL);
+  g_return_val_if_fail (ATK_IS_TABLE (table), 0);
 
-  iface = ATK_TABLE_GET_IFACE (obj);
+  iface = ATK_TABLE_GET_IFACE (table);
 
   if (iface->get_selected_columns)
-    return (iface->get_selected_columns) (obj);
+    return (iface->get_selected_columns) (table, selected);
   else
-    return NULL;
+    return 0;
 }
 
 /**
@@ -585,18 +631,17 @@ atk_table_get_selected_columns (AtkTable *obj)
  * if value does not implement this interface.
  **/
 gboolean
-atk_table_is_column_selected (AtkTable *obj,
+atk_table_is_column_selected (AtkTable *table,
                               gint     column)
 {
   AtkTableIface *iface;
 
-  g_return_val_if_fail (obj != NULL, FALSE);
-  g_return_val_if_fail (ATK_IS_TABLE (obj), FALSE);
+  g_return_val_if_fail (ATK_IS_TABLE (table), FALSE);
 
-  iface = ATK_TABLE_GET_IFACE (obj);
+  iface = ATK_TABLE_GET_IFACE (table);
 
   if (iface->is_column_selected)
-    return (iface->is_column_selected) (obj, column);
+    return (iface->is_column_selected) (table, column);
   else
     return FALSE;
 }
@@ -617,18 +662,17 @@ atk_table_is_column_selected (AtkTable *obj,
  * if value does not implement this interface.
  **/
 gboolean
-atk_table_is_row_selected (AtkTable *obj,
+atk_table_is_row_selected (AtkTable *table,
                            gint     row)
 {
   AtkTableIface *iface;
 
-  g_return_val_if_fail (obj != NULL, FALSE);
-  g_return_val_if_fail (ATK_IS_TABLE (obj), FALSE);
+  g_return_val_if_fail (ATK_IS_TABLE (table), FALSE);
 
-  iface = ATK_TABLE_GET_IFACE (obj);
+  iface = ATK_TABLE_GET_IFACE (table);
 
   if (iface->is_row_selected)
-    return (iface->is_row_selected) (obj, row);
+    return (iface->is_row_selected) (table, row);
   else
     return FALSE;
 }
@@ -639,7 +683,7 @@ atk_table_is_row_selected (AtkTable *obj,
  * @row: a #gint representing a row in @table
  * @column: a #gint representing a column in @table
  *
- * Gets a boolean value indicating whether the acessible object
+ * Gets a boolean value indicating whether the accessible object
  * at the specified @row and @column is selected
  * Note: callers should not rely on %NULL or on a zero value for
  * indication of whether AtkSelectionIface is implemented, they should
@@ -650,19 +694,135 @@ atk_table_is_row_selected (AtkTable *obj,
  * if value does not implement this interface.
  **/
 gboolean
-atk_table_is_selected (AtkTable *obj,
+atk_table_is_selected (AtkTable *table,
                        gint     row,
                        gint     column)
 {
   AtkTableIface *iface;
 
-  g_return_val_if_fail (obj != NULL, FALSE);
-  g_return_val_if_fail (ATK_IS_TABLE (obj), FALSE);
+  g_return_val_if_fail (ATK_IS_TABLE (table), FALSE);
 
-  iface = ATK_TABLE_GET_IFACE (obj);
+  iface = ATK_TABLE_GET_IFACE (table);
 
   if (iface->is_selected)
-    return (iface->is_selected) (obj, row, column);
+    return (iface->is_selected) (table, row, column);
+  else
+    return FALSE;
+}
+
+/**
+ * atk_table_add_row_selection:
+ * @table: a GObject instance that implements AtkTableIface
+ * @row: a #gint representing a row in @table
+ *
+ * Adds the specified @row to the selection. 
+ * Note: callers should not rely on %NULL or on a zero value for
+ * indication of whether AtkSelectionIface is implemented, they should
+ * use type checking/interface checking macros or the
+ * atk_get_accessible_table() convenience method.
+ *
+ * Returns: a gboolean representing if row was successfully added to selection,
+ * or 0 if value does not implement this interface.
+ **/
+gboolean
+atk_table_add_row_selection (AtkTable *table,
+                       		 gint     row)
+{
+  AtkTableIface *iface;
+
+  g_return_val_if_fail (ATK_IS_TABLE (table), FALSE);
+
+  iface = ATK_TABLE_GET_IFACE (table);
+
+  if (iface->add_row_selection)
+    return (iface->add_row_selection) (table, row);
+  else
+    return FALSE;
+}
+/**
+ * atk_table_remove_row_selection:
+ * @table: a GObject instance that implements AtkTableIface
+ * @row: a #gint representing a row in @table
+ *
+ * Removes the specified @row from the selection. 
+ * Note: callers should not rely on %NULL or on a zero value for
+ * indication of whether AtkSelectionIface is implemented, they should
+ * use type checking/interface checking macros or the
+ * atk_get_accessible_table() convenience method.
+ *
+ * Returns: a gboolean representing if the row was successfully removed from
+ * the selection, or 0 if value does not implement this interface.
+ **/
+gboolean
+atk_table_remove_row_selection (AtkTable *table,
+                       		    gint     row)
+{
+  AtkTableIface *iface;
+
+  g_return_val_if_fail (ATK_IS_TABLE (table), FALSE);
+
+  iface = ATK_TABLE_GET_IFACE (table);
+
+  if (iface->remove_row_selection)
+    return (iface->remove_row_selection) (table, row);
+  else
+    return FALSE;
+}
+/**
+ * atk_table_add_column_selection:
+ * @table: a GObject instance that implements AtkTableIface
+ * @column: a #gint representing a column in @table
+ *
+ * Adds the specified @column to the selection. 
+ * Note: callers should not rely on %NULL or on a zero value for
+ * indication of whether AtkSelectionIface is implemented, they should
+ * use type checking/interface checking macros or the
+ * atk_get_accessible_table() convenience method.
+ *
+ * Returns: a gboolean representing if the column was successfully added to 
+ * the selection, or 0 if value does not implement this interface.
+ **/
+gboolean
+atk_table_add_column_selection (AtkTable *table,
+                       		    gint     column)
+{
+  AtkTableIface *iface;
+
+  g_return_val_if_fail (ATK_IS_TABLE (table), FALSE);
+
+  iface = ATK_TABLE_GET_IFACE (table);
+
+  if (iface->add_column_selection)
+    return (iface->add_column_selection) (table, column);
+  else
+    return FALSE;
+}
+/**
+ * atk_table_remove_column_selection:
+ * @table: a GObject instance that implements AtkTableIface
+ * @column: a #gint representing a column in @table
+ *
+ * Adds the specified @column to the selection. 
+ * Note: callers should not rely on %NULL or on a zero value for
+ * indication of whether AtkSelectionIface is implemented, they should
+ * use type checking/interface checking macros or the
+ * atk_get_accessible_table() convenience method.
+ *
+ * Returns: a gboolean representing if the column was successfully removed from
+ * the selection, or 0 if value does not implement this interface.
+ **/
+gboolean
+atk_table_remove_column_selection (AtkTable *table,
+                       			   gint     column)
+{
+  AtkTableIface *iface;
+
+  g_return_val_if_fail (ATK_IS_TABLE (table), FALSE);
+
+  iface = ATK_TABLE_GET_IFACE (table);
+
+  if (iface->remove_column_selection)
+    return (iface->remove_column_selection) (table, column);
   else
     return FALSE;
 }
@@ -675,18 +835,17 @@ atk_table_is_selected (AtkTable *obj,
  * Sets the caption for the table.
  **/
 void
-atk_table_set_caption (AtkTable       *obj,
-                       gchar          *caption)
+atk_table_set_caption (AtkTable       *table,
+                       const gchar    *caption)
 {
   AtkTableIface *iface;
 
-  g_return_if_fail (obj != NULL);
-  g_return_if_fail (ATK_IS_TABLE (obj));
+  g_return_if_fail (ATK_IS_TABLE (table));
 
-  iface = ATK_TABLE_GET_IFACE (obj);
+  iface = ATK_TABLE_GET_IFACE (table);
 
   if (iface->set_caption)
-    (iface->set_caption) (obj, caption);
+    (iface->set_caption) (table, caption);
 }
 
 /**
@@ -699,19 +858,18 @@ atk_table_set_caption (AtkTable       *obj,
  * Sets the description text for the specified @column of the @table.
  **/
 void
-atk_table_set_column_description (AtkTable       *obj,
+atk_table_set_column_description (AtkTable       *table,
                                   gint           column,
-                                  gchar          *description)
+                                  const gchar    *description)
 {
   AtkTableIface *iface;
 
-  g_return_if_fail (obj != NULL);
-  g_return_if_fail (ATK_IS_TABLE (obj));
+  g_return_if_fail (ATK_IS_TABLE (table));
 
-  iface = ATK_TABLE_GET_IFACE (obj);
+  iface = ATK_TABLE_GET_IFACE (table);
 
   if (iface->set_column_description)
-    (iface->set_column_description) (obj, column, description);
+    (iface->set_column_description) (table, column, description);
 }
 
 /**
@@ -720,22 +878,21 @@ atk_table_set_column_description (AtkTable       *obj,
  * @column: a #gint representing a column in @table
  * @header: an #AtkTable
  *
- * Sets the specified column header to @header
+ * Sets the specified column header to @header.
  **/
 void
-atk_table_set_column_header (AtkTable  *obj,
+atk_table_set_column_header (AtkTable  *table,
                              gint      column,
                              AtkObject *header)
 {
   AtkTableIface *iface;
 
-  g_return_if_fail (obj != NULL);
-  g_return_if_fail (ATK_IS_TABLE (obj));
+  g_return_if_fail (ATK_IS_TABLE (table));
 
-  iface = ATK_TABLE_GET_IFACE (obj);
+  iface = ATK_TABLE_GET_IFACE (table);
 
   if (iface->set_column_header)
-    (iface->set_column_header) (obj, column, header);
+    (iface->set_column_header) (table, column, header);
 }
 
 /**
@@ -748,19 +905,18 @@ atk_table_set_column_header (AtkTable  *obj,
  * Sets the description text for the specified @row of @table.
  **/
 void
-atk_table_set_row_description (AtkTable       *obj,
+atk_table_set_row_description (AtkTable       *table,
                                gint           row,
-                               gchar          *description)
+                               const gchar    *description)
 {
   AtkTableIface *iface;
 
-  g_return_if_fail (obj != NULL);
-  g_return_if_fail (ATK_IS_TABLE (obj));
+  g_return_if_fail (ATK_IS_TABLE (table));
 
-  iface = ATK_TABLE_GET_IFACE (obj);
+  iface = ATK_TABLE_GET_IFACE (table);
 
   if (iface->set_row_description)
-    (iface->set_row_description) (obj, row, description);
+    (iface->set_row_description) (table, row, description);
 }
 
 /**
@@ -769,22 +925,21 @@ atk_table_set_row_description (AtkTable       *obj,
  * @row: a #gint representing a row in @table
  * @header: an #AtkTable 
  *
- * Sets the specified row header to @header
+ * Sets the specified row header to @header.
  **/
 void
-atk_table_set_row_header (AtkTable  *obj,
+atk_table_set_row_header (AtkTable  *table,
                           gint      row,
                           AtkObject *header)
 {
   AtkTableIface *iface;
 
-  g_return_if_fail (obj != NULL);
-  g_return_if_fail (ATK_IS_TABLE (obj));
+  g_return_if_fail (ATK_IS_TABLE (table));
 
-  iface = ATK_TABLE_GET_IFACE (obj);
+  iface = ATK_TABLE_GET_IFACE (table);
 
   if (iface->set_row_header)
-    (iface->set_row_header) (obj, row, header);
+    (iface->set_row_header) (table, row, header);
 }
 
 /**
@@ -793,19 +948,62 @@ atk_table_set_row_header (AtkTable  *obj,
  * @accessible: an #AtkObject representing the summary description
  * to set for @table
  *
- * Sets the summary description of the table
+ * Sets the summary description of the table.
  **/
 void
-atk_table_set_summary (AtkTable       *obj,
+atk_table_set_summary (AtkTable       *table,
                        AtkObject      *accessible)
 {
   AtkTableIface *iface;
 
-  g_return_if_fail (obj != NULL);
-  g_return_if_fail (ATK_IS_TABLE (obj));
+  g_return_if_fail (ATK_IS_TABLE (table));
 
-  iface = ATK_TABLE_GET_IFACE (obj);
+  iface = ATK_TABLE_GET_IFACE (table);
 
   if (iface->set_summary)
-    (iface->set_summary) (obj, accessible);
+    (iface->set_summary) (table, accessible);
+}
+
+static gint
+atk_table_real_get_index_at (AtkTable *table,
+                             gint     row,
+                             gint     column)
+{
+  gint n_cols, n_rows;
+
+  n_cols = atk_table_get_n_columns (table);
+  n_rows = atk_table_get_n_rows (table);
+
+  g_return_val_if_fail (row < n_rows, 0);
+  g_return_val_if_fail (column < n_cols, 0);
+
+  return row * n_cols + column;
+}
+
+static gint
+atk_table_real_get_column_at_index (AtkTable *table,
+                                    gint     index)
+{
+  gint n_cols;
+
+  n_cols = atk_table_get_n_columns (table);
+
+  if (n_cols == 0)
+    return 0;
+  else
+    return (gint) (index % n_cols);
+}
+
+static gint
+atk_table_real_get_row_at_index (AtkTable *table,
+                                 gint     index)
+{
+  gint n_cols;
+
+  n_cols = atk_table_get_n_columns (table);
+
+  if (n_cols == 0)
+    return 0;
+  else
+    return (gint) (index / n_cols);
 }
