@@ -277,12 +277,12 @@ gdk_window_procedure (HWND   hwnd,
 {
   LRESULT retval;
 
-  GDK_NOTE (EVENTS, g_print ("gdk_window_procedure: %#lx %s\n",
+  GDK_NOTE (MISC, g_print ("gdk_window_procedure: %#lx %s\n",
 			     (gulong) hwnd, gdk_win32_message_name (message)));
 
   retval = real_window_procedure (hwnd, message, wparam, lparam);
 
-  GDK_NOTE (EVENTS, g_print ("gdk_window_procedure: %#lx returns %ld\n",
+  GDK_NOTE (MISC, g_print ("gdk_window_procedure: %#lx returns %ld\n",
 			     (gulong) hwnd, retval));
 
   return retval;
@@ -1354,6 +1354,8 @@ propagate (GdkWindow  **window,
 	   gboolean   (*doesnt_want_it) (gint mask,
 					 MSG *msg))
 {
+  gboolean in_propagation = FALSE;
+
   if (grab_window != NULL && !grab_owner_events)
     {
       /* Event source is grabbed with owner_events FALSE */
@@ -1414,9 +1416,11 @@ propagate (GdkWindow  **window,
 	      gdk_drawable_unref (*window);
 	      *window = GDK_WINDOW (GDK_WINDOW_OBJECT (*window)->parent);
 	      gdk_drawable_ref (*window);
-	      GDK_NOTE (EVENTS, g_print ("...propagating to %#lx\n",
+	      GDK_NOTE (EVENTS, g_print ("%s %#lx",
+					 (in_propagation ? "," : " ...propagating to"),
 					 (gulong) GDK_WINDOW_HWND (*window)));
 	      /* The only branch where we actually continue the loop */
+	      in_propagation = TRUE;
 	    }
 	}
       else
@@ -1536,6 +1540,7 @@ gdk_event_translate (GdkEvent *event,
   GdkColormapPrivateWin32 *colormap_private;
   GdkPixmap *pixmap;
   GdkPixmapImplWin32 *pixmap_impl;
+  gint xoffset, yoffset;
 
   int button;
   int i, j;
@@ -1623,12 +1628,17 @@ gdk_event_translate (GdkEvent *event,
       if (result != GDK_FILTER_CONTINUE)
 	{
 	  return_val =  (result == GDK_FILTER_TRANSLATE) ? TRUE : FALSE;
-	  *ret_val_flagp = TRUE;
-	  *ret_valp = return_val;
+	  if (ret_val_flagp)
+	    *ret_val_flagp = TRUE;
+	  if (ret_valp)
+	    *ret_valp = return_val;
 	  goto done;
 	}
     }
 
+  /* to translate coordinates to the internal > 16 bit system */
+  _gdk_windowing_window_get_offsets (window, &xoffset, &yoffset);
+
   if (msg->message == gdk_selection_notify_msg)
     {
       GDK_NOTE (EVENTS, g_print ("gdk_selection_notify_msg: %#lx\n",
@@ -1670,6 +1680,7 @@ gdk_event_translate (GdkEvent *event,
       event->selection.type = GDK_SELECTION_CLEAR;
       event->selection.window = window;
       event->selection.selection = msg->wParam;
+      event->selection.target = msg->lParam;
       event->selection.time = msg->time;
 
       return_val = !GDK_WINDOW_DESTROYED (window);
@@ -1723,8 +1734,8 @@ gdk_event_translate (GdkEvent *event,
 	GDK_SCROLL_UP : GDK_SCROLL_DOWN;
       event->scroll.window = window;
       event->scroll.time = msg->time;
-      event->scroll.x = (gint16) pt.x;
-      event->scroll.y = (gint16) pt.y;
+      event->scroll.x = (gint16) pt.x + xoffset;
+      event->scroll.y = (gint16) pt.y + yoffset;
       event->scroll.x_root = (gint16) LOWORD (msg->lParam);
       event->scroll.y_root = (gint16) HIWORD (msg->lParam);
       event->scroll.state = 0;	/* No state information with MSH_MOUSEWHEEL */
@@ -2236,6 +2247,8 @@ gdk_event_translate (GdkEvent *event,
 	translate_mouse_coords (orig_window, window, msg);
       event->button.x = current_x = (gint16) LOWORD (msg->lParam);
       event->button.y = current_y = (gint16) HIWORD (msg->lParam);
+      event->button.x += xoffset;  /* XXX translate current_x, y too? */
+      event->button.y += yoffset;
       event->button.x_root = msg->pt.x;
       event->button.y_root = msg->pt.y;
       event->button.axes = NULL;
@@ -2290,8 +2303,8 @@ gdk_event_translate (GdkEvent *event,
 	  event->button.time = msg->time;
 	  if (window != orig_window)
 	    translate_mouse_coords (orig_window, window, msg);
-	  event->button.x = (gint16) LOWORD (msg->lParam);
-	  event->button.y = (gint16) HIWORD (msg->lParam);
+	  event->button.x = (gint16) LOWORD (msg->lParam) + xoffset;
+	  event->button.y = (gint16) HIWORD (msg->lParam) + yoffset;
 	  event->button.x_root = msg->pt.x;
 	  event->button.y_root = msg->pt.y;
 	  event->button.axes = NULL;
@@ -2354,6 +2367,8 @@ gdk_event_translate (GdkEvent *event,
 	translate_mouse_coords (orig_window, window, msg);
       event->motion.x = current_x = (gint16) LOWORD (msg->lParam);
       event->motion.y = current_y = (gint16) HIWORD (msg->lParam);
+      event->motion.x += xoffset;
+      event->motion.y += yoffset;
       event->motion.x_root = current_x_root = msg->pt.x;
       event->motion.y_root = current_y_root = msg->pt.y;
       event->motion.axes = NULL;
@@ -2379,8 +2394,8 @@ gdk_event_translate (GdkEvent *event,
 	  event->crossing.window = current_window;
 	  event->crossing.subwindow = NULL;
 	  event->crossing.time = msg->time;
-	  event->crossing.x = current_x;
-	  event->crossing.y = current_y;
+	  event->crossing.x = current_x + xoffset; /* XXX translated current_x */
+	  event->crossing.y = current_y + yoffset;
 	  event->crossing.x_root = current_x_root;
 	  event->crossing.y_root = current_y_root;
 	  event->crossing.mode = GDK_CROSSING_NORMAL;
@@ -2446,8 +2461,8 @@ gdk_event_translate (GdkEvent *event,
 	GDK_SCROLL_UP : GDK_SCROLL_DOWN;
       event->scroll.window = window;
       event->scroll.time = msg->time;
-      event->scroll.x = (gint16) pt.x;
-      event->scroll.y = (gint16) pt.y;
+      event->scroll.x = (gint16) pt.x + xoffset;
+      event->scroll.y = (gint16) pt.y + yoffset;
       event->scroll.x_root = (gint16) LOWORD (msg->lParam);
       event->scroll.y_root = (gint16) HIWORD (msg->lParam);
       event->scroll.state = build_pointer_event_state (msg);
@@ -2467,8 +2482,8 @@ gdk_event_translate (GdkEvent *event,
       event->crossing.window = window;
       event->crossing.subwindow = NULL;
       event->crossing.time = msg->time;
-      event->crossing.x = current_x;
-      event->crossing.y = current_y;
+      event->crossing.x = current_x + xoffset; /* XXX translated current_x */
+      event->crossing.y = current_y + yoffset;
       event->crossing.x_root = current_xroot;
       event->crossing.y_root = current_yroot;
       event->crossing.mode = GDK_CROSSING_NORMAL;
@@ -2517,6 +2532,9 @@ gdk_event_translate (GdkEvent *event,
       if (GDK_WINDOW_DESTROYED (window))
 	break;
 
+      *ret_val_flagp = TRUE; /* always claim as handled */
+      *ret_valp = 1;
+
       if (GDK_WINDOW_OBJECT (window)->input_only)
 	break;
 
@@ -2546,8 +2564,6 @@ gdk_event_translate (GdkEvent *event,
 		   colormap_private->xcolormap->palette, k);
 #endif
 	}
-      *ret_val_flagp = TRUE;
-      *ret_valp = 1;
 
       if (GDK_WINDOW_OBJECT (window)->bg_pixmap == GDK_PARENT_RELATIVE_BG)
 	{
@@ -2563,6 +2579,14 @@ gdk_event_translate (GdkEvent *event,
 	    }
 	}
 
+      if (GDK_WINDOW_IMPL_WIN32 (GDK_WINDOW_OBJECT (window)->impl)->position_info.no_bg)
+	{
+	  /* improves scolling effect, e.g. main buttons of testgtk */
+	  *ret_val_flagp = TRUE;
+	  *ret_valp = 1;
+	  break;
+	}
+
       if (GDK_WINDOW_OBJECT (window)->bg_pixmap == NULL)
 	{
 	  bg = gdk_colormap_color (GDK_DRAWABLE_IMPL_WIN32 (GDK_WINDOW_OBJECT (window)->impl)->colormap,
@@ -2664,12 +2688,6 @@ gdk_event_translate (GdkEvent *event,
           break;
         }
 
-      /* HB: don't generate GDK_EXPOSE events for InputOnly
-       * windows -> backing store now works!
-       */
-      if (GDK_WINDOW_OBJECT (window)->input_only)
-	break;
-
       hdc = BeginPaint (msg->hwnd, &paintstruct);
 
       GDK_NOTE (EVENTS,
@@ -2683,9 +2701,18 @@ gdk_event_translate (GdkEvent *event,
 
       EndPaint (msg->hwnd, &paintstruct);
 
+      /* HB: don't generate GDK_EXPOSE events for InputOnly
+       * windows -> backing store now works!
+       */
+      if (GDK_WINDOW_OBJECT (window)->input_only)
+	break;
+
       if (!(window_impl->event_mask & GDK_EXPOSURE_MASK))
 	break;
 
+      if (GDK_WINDOW_OBJECT (window)->bg_pixmap == GDK_NO_BG)
+	break;
+
       if ((paintstruct.rcPaint.right == paintstruct.rcPaint.left)
           || (paintstruct.rcPaint.bottom == paintstruct.rcPaint.top))
         break;
@@ -2720,12 +2747,12 @@ gdk_event_translate (GdkEvent *event,
         {
           GdkRectangle expose_rect;
 
-          expose_rect.x = paintstruct.rcPaint.left;
-          expose_rect.y = paintstruct.rcPaint.top;
+          expose_rect.x = paintstruct.rcPaint.left + xoffset;
+          expose_rect.y = paintstruct.rcPaint.top + yoffset;
           expose_rect.width = paintstruct.rcPaint.right - paintstruct.rcPaint.left;
           expose_rect.height = paintstruct.rcPaint.bottom - paintstruct.rcPaint.top;
 
-	    _gdk_window_process_expose (window, GetMessageTime (), &expose_rect);
+	    _gdk_window_process_expose (window, msg->time, &expose_rect);
 
 	    return_val = FALSE;
         }
@@ -2851,9 +2878,11 @@ gdk_event_translate (GdkEvent *event,
 	{
 	  mmi->ptMaxTrackSize.x = window_impl->hint_max_width;
 	  mmi->ptMaxTrackSize.y = window_impl->hint_max_height;
-	    
-	  mmi->ptMaxSize.x = window_impl->hint_max_width;
-	  mmi->ptMaxSize.y = window_impl->hint_max_height;
+
+	  /* kind of WM functionality, limit maximized size to screen */
+	  mmi->ptMaxPosition.x = 0; mmi->ptMaxPosition.y = 0;	    
+	  mmi->ptMaxSize.x = MIN(window_impl->hint_max_width, gdk_screen_width ());
+	  mmi->ptMaxSize.y = MIN(window_impl->hint_max_height, gdk_screen_height ());
 	}
       break;
 
@@ -2884,7 +2913,44 @@ gdk_event_translate (GdkEvent *event,
 	  return_val = !GDK_WINDOW_DESTROYED (window);
 	}
       break;
+#if 0 /* not quite right, otherwise it may be faster/better than WM_(MOVE|SIZE) 
+       * remove decoration (frame) sizes ?
+       */
+    case WM_WINDOWPOSCHANGED :
 
+      if (!(window_impl->event_mask & GDK_STRUCTURE_MASK))
+	break;
+
+      if (GDK_WINDOW_TYPE (window) != GDK_WINDOW_CHILD
+	  && !IsIconic(msg->hwnd)
+          && IsWindowVisible(msg->hwnd))
+	{
+	  LPWINDOWPOS lpwp = (LPWINDOWPOS) (msg->lParam);
+
+	  event->configure.type = GDK_CONFIGURE;
+	  event->configure.window = window;
+	  event->configure.x = lpwp->x;
+	  event->configure.y = lpwp->y;
+	  event->configure.width = lpwp->cx;
+	  event->configure.height = lpwp->cy;
+	  GDK_WINDOW_OBJECT (window)->x = event->configure.x;
+	  GDK_WINDOW_OBJECT (window)->y = event->configure.y;
+	  window_impl->width = event->configure.width;
+	  window_impl->height = event->configure.height;
+	  
+	  return_val = !GDK_WINDOW_DESTROYED (window);
+
+          GDK_NOTE (EVENTS, g_print ("WM_WINDOWPOSCHANGED: %#lx  %ldx%ld@+%ld+%ld\n",
+				     (gulong) msg->hwnd,
+				     lpwp->cx, lpwp->cy, lpwp->x, lpwp->y));
+
+	  if (ret_val_flagp)
+	    *ret_val_flagp = TRUE;
+	  if (ret_valp)
+	    *ret_valp = 0;
+	}
+      break;
+#endif
     case WM_CLOSE:
       GDK_NOTE (EVENTS, g_print ("WM_CLOSE: %#lx\n", (gulong) msg->hwnd));
 
@@ -3043,6 +3109,8 @@ void
 gdk_events_queue (void)
 {
   MSG msg;
+  GdkEvent *event;
+  GList *node;
 
   while (!gdk_event_queue_find_first ()
 	 && PeekMessage (&msg, NULL, 0, 0, PM_REMOVE))
@@ -3054,7 +3122,33 @@ gdk_events_queue (void)
 	  || (active_imm_msgpump_owner->lpVtbl->OnTranslateMessage) (active_imm_msgpump_owner, &msg) != S_OK)
 	TranslateMessage (&msg);
 
+#if 1 /* It was like this all the time */
       DispatchMessage (&msg);
+#else /* but this one is more similar to the X implementation. Any effect ? */
+      event = gdk_event_new ();
+      
+      event->any.type = GDK_NOTHING;
+      event->any.window = NULL;
+      event->any.send_event = InSendMessage ();
+
+      ((GdkEventPrivate *)event)->flags |= GDK_EVENT_PENDING;
+
+      gdk_event_queue_append (event);
+      node = gdk_queued_tail;
+
+      if (gdk_event_translate (event, &msg, NULL, NULL, FALSE))
+	{
+	  ((GdkEventPrivate *)event)->flags &= ~GDK_EVENT_PENDING;
+	}
+      else
+	{
+	  gdk_event_queue_remove_link (node);
+	  g_list_free_1 (node);
+	  gdk_event_free (event);
+        DispatchMessage (&msg);
+	}
+
+#endif
     }
 }
 
@@ -3146,6 +3240,7 @@ gdk_flush (void)
       TranslateMessage (&msg); /* Translate virt. key codes */
       DispatchMessage (&msg);  /* Dispatch msg. to window */
     }
+
   GdiFlush ();
 }
 
