@@ -438,13 +438,31 @@ gdk_event_translate (GdkEvent *event,
   gint xoffset, yoffset;
   
   return_val = FALSE;
+
+  /* init these, since the done: block uses them */
+  window = NULL;
+  window_private = NULL;
+  event->any.window = NULL;
   
-  /* Find the GdkWindow that this event occurred in.
-   * 
-   * We handle events with window=None
-   *  specially - they are generated by XFree86's XInput under
-   *  some circumstances.
-   */
+  if (gdk_default_filters)
+    {
+      /* Apply global filters */
+      GdkFilterReturn result;
+      result = gdk_event_apply_filters (xevent, event,
+                                        gdk_default_filters);
+      
+      if (result != GDK_FILTER_CONTINUE)
+        {
+          return_val = (result == GDK_FILTER_TRANSLATE) ? TRUE : FALSE;
+          goto done;
+        }
+    }  
+
+   /* We handle events with window=None
+    *  specially - they are generated by XFree86's XInput under
+    *  some circumstances. This handling for obvious reasons
+    * goes before we bother to lookup the event window.
+    */
   
   if (xevent->xany.window == None)
     {
@@ -455,53 +473,40 @@ gdk_event_translate (GdkEvent *event,
       else
 	return_val = FALSE;
     }
+
+  /* Find the GdkWindow that this event occurred in. */
   
   window = gdk_window_lookup (xevent->xany.window);
   window_private = (GdkWindowObject *) window;
-
-  if (_gdk_moveresize_window &&
-      (xevent->xany.type == MotionNotify ||
-       xevent->xany.type == ButtonRelease))
-    {
-      _gdk_moveresize_handle_event (xevent);
-      gdk_window_unref (window);
-      return FALSE;
-    }
-    
-  if (wmspec_check_window != None &&
-      xevent->xany.window == wmspec_check_window)
-    {
-      if (xevent->type == DestroyNotify)
-        wmspec_check_window = None;
-      
-      /* Eat events on this window unless someone had wrapped
-       * it as a foreign window
-       */
-      if (window == NULL)
-        return FALSE;
-    }
   
-  /* FIXME: window might be a GdkPixmap!!! */
   if (window != NULL)
     {
-      window_impl = GDK_WINDOW_IMPL_X11 (window_private->impl);
+      /* Window may be a pixmap, so check its type.
+       * (This check is probably too expensive unless
+       *  GLib short-circuits an exact type match,
+       *  which has been proposed)
+       */
+      if (GDK_IS_WINDOW (window))
+        {
+          window_impl = GDK_WINDOW_IMPL_X11 (window_private->impl);
 	  
-      if (xevent->xany.window != GDK_WINDOW_XID (window))
-	{
-	  g_assert (xevent->xany.window == window_impl->focus_window);
+          if (xevent->xany.window != GDK_WINDOW_XID (window))
+            {
+              g_assert (xevent->xany.window == window_impl->focus_window);
+              
+              switch (xevent->type)
+                {
+                case KeyPress:
+                case KeyRelease:
+                  xevent->xany.window = GDK_WINDOW_XID (window);
+                  break;
+                default:
+                  return False;
+                }
+            }
+        }
 
-	  switch (xevent->type)
-	    {
-	    case KeyPress:
-	    case KeyRelease:
-	      xevent->xany.window = GDK_WINDOW_XID (window);
-	      break;
-	    default:
-	      return False;
-	    }
-	}
-
-      gdk_window_ref (window);
+      g_object_ref (G_OBJECT (window));
     }
 
   event->any.window = window;
@@ -512,15 +517,12 @@ gdk_event_translate (GdkEvent *event,
       if (xevent->type != DestroyNotify)
         return FALSE;
     }
-  else
+  else if (window_private)
     {
-      /* Check for filters for this window
-       */
+      /* Apply per-window filters */
       GdkFilterReturn result;
       result = gdk_event_apply_filters (xevent, event,
-					window_private
-					?window_private->filters
-					:gdk_default_filters);
+					window_private->filters);
       
       if (result != GDK_FILTER_CONTINUE)
 	{
@@ -528,7 +530,34 @@ gdk_event_translate (GdkEvent *event,
 	  goto done;
 	}
     }
+      
+  if (wmspec_check_window != None &&
+      xevent->xany.window == wmspec_check_window)
+    {
+      if (xevent->type == DestroyNotify)
+        wmspec_check_window = None;
+      
+      /* Eat events on this window unless someone had wrapped
+       * it as a foreign window
+       */
+      if (window == NULL)
+	{
+	  return_val = FALSE;
+	  goto done;
+	}
+    }
 
+  if (window &&
+      _gdk_moveresize_window &&
+      (xevent->xany.type == MotionNotify ||
+       xevent->xany.type == ButtonRelease))
+    {
+      _gdk_moveresize_handle_event (xevent);
+      
+      return_val = FALSE;
+      goto done;
+    }
+  
   /* We do a "manual" conversion of the XEvent to a
    *  GdkEvent. The structures are mostly the same so
    *  the conversion is fairly straightforward. We also
@@ -547,10 +576,16 @@ gdk_event_translate (GdkEvent *event,
       xoffset = 0;
       yoffset = 0;
     }
-
+  
   switch (xevent->type)
     {
     case KeyPress:
+      if (window_private == NULL)
+        {
+          return_val = FALSE;
+          break;
+        }
+      
       /* Lookup the string corresponding to the given keysym.
        */
 
@@ -593,6 +628,12 @@ gdk_event_translate (GdkEvent *event,
       break;
       
     case KeyRelease:
+      if (window_private == NULL)
+        {
+          return_val = FALSE;
+          break;
+        }
+      
       /* Lookup the string corresponding to the given keysym.
        */
 
@@ -640,9 +681,9 @@ gdk_event_translate (GdkEvent *event,
 			   xevent->xbutton.x, xevent->xbutton.y,
 			   xevent->xbutton.button));
       
-      if (window_private &&
-	  (window_private->extension_events != 0) &&
-	  gdk_input_ignore_core)
+      if (window_private == NULL || 
+	  ((window_private->extension_events != 0) &&
+           gdk_input_ignore_core))
 	{
 	  return_val = FALSE;
 	  break;
@@ -703,9 +744,9 @@ gdk_event_translate (GdkEvent *event,
 			   xevent->xbutton.x, xevent->xbutton.y,
 			   xevent->xbutton.button));
       
-      if (window_private &&
-	  (window_private->extension_events != 0) &&
-	  gdk_input_ignore_core)
+      if (window_private == NULL ||
+	  ((window_private->extension_events != 0) &&
+           gdk_input_ignore_core))
 	{
 	  return_val = FALSE;
 	  break;
@@ -740,9 +781,9 @@ gdk_event_translate (GdkEvent *event,
 			   xevent->xmotion.x, xevent->xmotion.y,
 			   (xevent->xmotion.is_hint) ? "true" : "false"));
       
-      if (window_private &&
-	  (window_private->extension_events != 0) &&
-	  gdk_input_ignore_core)
+      if (window_private == NULL ||
+	  ((window_private->extension_events != 0) &&
+           gdk_input_ignore_core))
 	{
 	  return_val = FALSE;
 	  break;
@@ -768,7 +809,13 @@ gdk_event_translate (GdkEvent *event,
 			   xevent->xcrossing.window,
 			   xevent->xcrossing.detail,
 			   xevent->xcrossing.subwindow));
-
+ 
+      if (window_private == NULL)
+        {
+          return_val = FALSE;
+          break;
+        }
+      
       /* Handle focusing (in the case where no window manager is running */
       if (window &&
 	  GDK_WINDOW_TYPE (window) != GDK_WINDOW_CHILD &&
@@ -855,6 +902,12 @@ gdk_event_translate (GdkEvent *event,
 		g_message ("leave notify:\t\twindow: %ld  detail: %d subwin: %ld",
 			   xevent->xcrossing.window,
 			   xevent->xcrossing.detail, xevent->xcrossing.subwindow));
+
+      if (window_private == NULL)
+        {
+          return_val = FALSE;
+          break;
+        }
       
       /* Handle focusing (in the case where no window manager is running */
       if (window &&
@@ -1017,6 +1070,13 @@ gdk_event_translate (GdkEvent *event,
 			   xevent->xexpose.x, xevent->xexpose.y,
 			   xevent->xexpose.width, xevent->xexpose.height,
 			   event->any.send_event ? " (send)" : ""));
+      
+      if (window_private == NULL)
+        {
+          return_val = FALSE;
+          break;
+        }
+      
       {
 	GdkRectangle expose_rect;
 
@@ -1054,7 +1114,13 @@ gdk_event_translate (GdkEvent *event,
         GDK_NOTE (EVENTS,
 		  g_message ("graphics expose:\tdrawable: %ld",
 			     xevent->xgraphicsexpose.drawable));
-
+ 
+        if (window_private == NULL)
+          {
+            return_val = FALSE;
+            break;
+          }
+        
 	expose_rect.x = xevent->xgraphicsexpose.x + xoffset;
 	expose_rect.y = xevent->xgraphicsexpose.y + yoffset;
 	expose_rect.width = xevent->xgraphicsexpose.width;
@@ -1110,6 +1176,12 @@ gdk_event_translate (GdkEvent *event,
 	  }
 #endif /* G_ENABLE_DEBUG */
       
+      if (window_private == NULL)
+        {
+          return_val = FALSE;
+          break;
+        }
+      
       event->visibility.type = GDK_VISIBILITY_NOTIFY;
       event->visibility.window = window;
       
@@ -1148,14 +1220,20 @@ gdk_event_translate (GdkEvent *event,
       GDK_NOTE (EVENTS,
 		g_message ("destroy notify:\twindow: %ld",
 			   xevent->xdestroywindow.window));
-      
-      event->any.type = GDK_DESTROY;
-      event->any.window = window;
-      
-      return_val = window_private && !GDK_WINDOW_DESTROYED (window);
-      
-      if (window && GDK_WINDOW_XID (window) != GDK_ROOT_WINDOW())
-	gdk_window_destroy_notify (window);
+
+      /* Ignore DestroyNotify from SubstructureNotifyMask */
+      if (xevent->xdestroywindow.window == xevent->xdestroywindow.event)
+	{
+	  event->any.type = GDK_DESTROY;
+	  event->any.window = window;
+	  
+	  return_val = window_private && !GDK_WINDOW_DESTROYED (window);
+	  
+	  if (window && GDK_WINDOW_XID (window) != GDK_ROOT_WINDOW())
+	    gdk_window_destroy_notify (window);
+	}
+      else
+	return_val = FALSE;
       break;
       
     case UnmapNotify:
@@ -1231,7 +1309,9 @@ gdk_event_translate (GdkEvent *event,
 	  (window_private->extension_events != 0))
 	_gdk_input_configure_event (&xevent->xconfigure, window);
 
-      if (!window || GDK_WINDOW_TYPE (window) == GDK_WINDOW_CHILD)
+      if (!window ||
+          GDK_WINDOW_TYPE (window) == GDK_WINDOW_CHILD ||
+          GDK_WINDOW_TYPE (window) == GDK_WINDOW_ROOT)
 	return_val = FALSE;
       else
 	{
@@ -1240,8 +1320,7 @@ gdk_event_translate (GdkEvent *event,
 	  event->configure.width = xevent->xconfigure.width;
 	  event->configure.height = xevent->xconfigure.height;
 	  
-	  if (!xevent->xconfigure.x &&
-	      !xevent->xconfigure.y &&
+	  if (!xevent->xconfigure.send_event && 
 	      !GDK_WINDOW_DESTROYED (window))
 	    {
 	      gint tx = 0;
@@ -1296,6 +1375,12 @@ gdk_event_translate (GdkEvent *event,
 			   atom ? "\"" : "");
 		g_free (atom);
 		);
+
+      if (window_private == NULL)
+        {
+          return_val = FALSE;
+          break;
+        }
       
       event->property.type = GDK_PROPERTY_NOTIFY;
       event->property.window = window;
@@ -1406,13 +1491,21 @@ gdk_event_translate (GdkEvent *event,
 	    break;
 	  case GDK_FILTER_CONTINUE:
 	    /* Send unknown ClientMessage's on to Gtk for it to use */
-	    event->client.type = GDK_CLIENT_EVENT;
-	    event->client.window = window;
-	    event->client.message_type = xevent->xclient.message_type;
-	    event->client.data_format = xevent->xclient.format;
-	    memcpy(&event->client.data, &xevent->xclient.data,
-		   sizeof(event->client.data));
-	  }
+            if (window_private == NULL)
+              {
+                return_val = FALSE;
+              }
+            else
+              {
+                event->client.type = GDK_CLIENT_EVENT;
+                event->client.window = window;
+                event->client.message_type = xevent->xclient.message_type;
+                event->client.data_format = xevent->xclient.format;
+                memcpy(&event->client.data, &xevent->xclient.data,
+                       sizeof(event->client.data));
+              }
+            break;
+          }
       }
       
       break;
@@ -1945,7 +2038,9 @@ static struct
   { "Gtk/ToolbarStyle", "gtk-toolbar-style" },
   { "Gtk/ToolbarIconSize", "gtk-toolbar-icon-size" },
   { "Net/CursorBlink", "gtk-cursor-blink" },
-  { "Net/CursorBlinkTime", "gtk-cursor-blink-time" }
+  { "Net/CursorBlinkTime", "gtk-cursor-blink-time" },
+  { "Net/ThemeName", "gtk-theme-name" },
+  { "Gtk/KeyThemeName", "gtk-key-theme-name" }
 };
 
 static void
