@@ -193,7 +193,7 @@ gdk_window_impl_x11_get_colormap (GdkDrawable *drawable)
       drawable_impl->colormap == NULL)
     {
       XWindowAttributes window_attributes;
-      
+
       XGetWindowAttributes (drawable_impl->xdisplay,
                             drawable_impl->xid,
                             &window_attributes);
@@ -366,7 +366,21 @@ gdk_window_new (GdkWindow     *parent,
   private->y = y;
   impl->width = (attributes->width > 1) ? (attributes->width) : (1);
   impl->height = (attributes->height > 1) ? (attributes->height) : (1);
-  private->window_type = attributes->window_type;
+
+  if (attributes->wclass == GDK_INPUT_ONLY)
+    {
+      /* Backwards compatiblity - we've always ignored
+       * attributes->window_type for input-only windows
+       * before
+       */
+      if (GDK_WINDOW_TYPE (parent) == GDK_WINDOW_ROOT &&
+	  GDK_WINDOW_TYPE (parent) == GDK_WINDOW_FOREIGN)
+	private->window_type = GDK_WINDOW_TEMP;
+      else
+	private->window_type = GDK_WINDOW_CHILD;
+    }
+  else
+    private->window_type = attributes->window_type;
 
   _gdk_window_init_position (GDK_WINDOW (private));
   if (impl->position_info.big)
@@ -403,6 +417,26 @@ gdk_window_new (GdkWindow     *parent,
       xattributes_mask |= CWWinGravity;
     }
   
+  /* Sanity checks */
+  switch (private->window_type)
+    {
+    case GDK_WINDOW_TOPLEVEL:
+    case GDK_WINDOW_DIALOG:
+    case GDK_WINDOW_TEMP:
+      if (GDK_WINDOW_TYPE (parent) != GDK_WINDOW_ROOT &&
+	  GDK_WINDOW_TYPE (parent) != GDK_WINDOW_FOREIGN)
+	{
+	  g_warning (G_STRLOC "Toplevel windows must be created as children of\n"
+		     "of a window of type GDK_WINDOW_ROOT or GDK_WINDOW_FOREIGN");
+	  xparent = gdk_root_window;
+	}
+    case GDK_WINDOW_CHILD:
+      break;
+    default:
+      g_warning (G_STRLOC "cannot make windows of type %d", private->window_type);
+      return NULL;
+    }
+	  
   if (attributes->wclass == GDK_INPUT_OUTPUT)
     {
       class = InputOutput;
@@ -445,42 +479,16 @@ gdk_window_new (GdkWindow     *parent,
 	xattributes.bit_gravity = NorthWestGravity;
       
       xattributes_mask |= CWBitGravity;
-  
-      switch (private->window_type)
+
+      xattributes.colormap = GDK_COLORMAP_XCOLORMAP (draw_impl->colormap);
+      xattributes_mask |= CWColormap;
+
+      if (private->window_type == GDK_WINDOW_TEMP)
 	{
-	case GDK_WINDOW_TOPLEVEL:
-	  xattributes.colormap = GDK_COLORMAP_XCOLORMAP (draw_impl->colormap);
-	  xattributes_mask |= CWColormap;
-	  
-	  xparent = gdk_root_window;
-	  break;
-	  
-	case GDK_WINDOW_CHILD:
-	  xattributes.colormap = GDK_COLORMAP_XCOLORMAP (draw_impl->colormap);
-	  xattributes_mask |= CWColormap;
-	  break;
-	  
-	case GDK_WINDOW_DIALOG:
-	  xattributes.colormap = GDK_COLORMAP_XCOLORMAP (draw_impl->colormap);
-	  xattributes_mask |= CWColormap;
-	  
-	  xparent = gdk_root_window;
-	  break;
-	  
-	case GDK_WINDOW_TEMP:
-	  xattributes.colormap = GDK_COLORMAP_XCOLORMAP (draw_impl->colormap);
-	  xattributes_mask |= CWColormap;
-	  
-	  xparent = gdk_root_window;
-	  
 	  xattributes.save_under = True;
 	  xattributes.override_redirect = True;
 	  xattributes.cursor = None;
 	  xattributes_mask |= CWSaveUnder | CWOverrideRedirect;
-	  break;
-	case GDK_WINDOW_ROOT:
-	  g_error ("cannot make windows of type GDK_WINDOW_ROOT");
-	  break;
 	}
     }
   else
@@ -807,8 +815,9 @@ set_initial_hints (GdkWindow *window)
     }
 }
 
-void
-gdk_window_show (GdkWindow *window)
+static void
+show_window_internal (GdkWindow *window,
+                      gboolean   raise)
 {
   GdkWindowObject *private;
   
@@ -817,8 +826,9 @@ gdk_window_show (GdkWindow *window)
   private = (GdkWindowObject*) window;
   if (!private->destroyed)
     {
-      XRaiseWindow (GDK_WINDOW_XDISPLAY (window),
-		    GDK_WINDOW_XID (window));
+      if (raise)
+        XRaiseWindow (GDK_WINDOW_XDISPLAY (window),
+                      GDK_WINDOW_XID (window));
 
       if (!GDK_WINDOW_IS_MAPPED (window))
         {
@@ -837,6 +847,31 @@ gdk_window_show (GdkWindow *window)
     }
 }
 
+/**
+ * gdk_window_show_unraised:
+ * @window: a #GdkWindow
+ *
+ * Shows a #GdkWindow onscreen, but does not modify its stacking
+ * order. In contrast, gdk_window_show() will raise the window
+ * to the top of the window stack.
+ * 
+ **/
+void
+gdk_window_show_unraised (GdkWindow *window)
+{
+  g_return_if_fail (GDK_IS_WINDOW (window));
+  
+  show_window_internal (window, FALSE);
+}
+
+void
+gdk_window_show (GdkWindow *window)
+{
+  g_return_if_fail (GDK_IS_WINDOW (window));
+
+  show_window_internal (window, TRUE);
+}
+
 void
 gdk_window_hide (GdkWindow *window)
 {
@@ -1013,6 +1048,7 @@ gdk_window_reparent (GdkWindow *window,
   g_return_if_fail (window != NULL);
   g_return_if_fail (GDK_IS_WINDOW (window));
   g_return_if_fail (new_parent == NULL || GDK_IS_WINDOW (new_parent));
+  g_return_if_fail (GDK_WINDOW_TYPE (window) != GDK_WINDOW_ROOT);
   
   if (!new_parent)
     new_parent = gdk_parent_root;
@@ -1028,7 +1064,37 @@ gdk_window_reparent (GdkWindow *window,
 		     x, y);
   
   window_private->parent = (GdkWindowObject *)new_parent;
-  
+
+  /* Switch the window type as appropriate */
+
+  switch (GDK_WINDOW_TYPE (new_parent))
+    {
+    case GDK_WINDOW_ROOT:
+    case GDK_WINDOW_FOREIGN:
+      /* Now a toplevel */
+      if (GDK_WINDOW_TYPE (window) == GDK_WINDOW_CHILD)
+	{
+	  GDK_WINDOW_TYPE (window) = GDK_WINDOW_TOPLEVEL;
+	  XSetWMProtocols (GDK_WINDOW_XDISPLAY (window),
+			   GDK_WINDOW_XID (window),
+			   gdk_wm_window_protocols, 3);
+	}
+    case GDK_WINDOW_TOPLEVEL:
+    case GDK_WINDOW_CHILD:
+    case GDK_WINDOW_DIALOG:
+    case GDK_WINDOW_TEMP:
+      if (GDK_WINDOW_TYPE (window) != GDK_WINDOW_CHILD &&
+	  GDK_WINDOW_TYPE (window) != GDK_WINDOW_FOREIGN)
+	{
+	  /* If we were being sophisticated, we'd save the old window type
+	   * here, and restore it if we were reparented back to the
+	   * toplevel. However, the difference between different types
+	   * of toplevels only really matters on creation anyways.
+	   */
+	  GDK_WINDOW_TYPE (window) = GDK_WINDOW_CHILD;
+	}
+    }
+
   if (old_parent_private)
     old_parent_private->children = g_list_remove (old_parent_private->children, window);
   
@@ -1241,12 +1307,10 @@ gdk_wmspec_change_state (gboolean add,
 			 GdkAtom state2)
 {
   XEvent xev;
-  Atom op;
 
-  if (add)
-    op = gdk_atom_intern ("_NET_WM_STATE_ADD", FALSE);
-  else
-    op = gdk_atom_intern ("_NET_WM_STATE_REMOVE", FALSE);
+#define _NET_WM_STATE_REMOVE        0    /* remove/unset property */
+#define _NET_WM_STATE_ADD           1    /* add/set property */
+#define _NET_WM_STATE_TOGGLE        2    /* toggle property  */  
   
   xev.xclient.type = ClientMessage;
   xev.xclient.serial = 0;
@@ -1255,7 +1319,7 @@ gdk_wmspec_change_state (gboolean add,
   xev.xclient.window = GDK_WINDOW_XID (window);
   xev.xclient.message_type = gdk_atom_intern ("_NET_WM_STATE", FALSE);
   xev.xclient.format = 32;
-  xev.xclient.data.l[0] = op;
+  xev.xclient.data.l[0] = add ? _NET_WM_STATE_ADD : _NET_WM_STATE_REMOVE;
   xev.xclient.data.l[1] = state1;
   xev.xclient.data.l[2] = state2;
   
@@ -1323,6 +1387,16 @@ gdk_window_set_geometry_hints (GdkWindow      *window,
       size_hints.x = 0;
       size_hints.y = 0;
     }
+
+  if (geom_mask & GDK_HINT_USER_POS)
+    {
+      size_hints.flags |= USPosition;
+    }
+
+  if (geom_mask & GDK_HINT_USER_SIZE)
+    {
+      size_hints.flags |= USSize;
+    }
   
   if (geom_mask & GDK_HINT_MIN_SIZE)
     {
@@ -1926,10 +2000,10 @@ gdk_window_get_frame_extents (GdkWindow    *window,
 }
 
 GdkWindow*
-gdk_window_get_pointer (GdkWindow       *window,
-			gint            *x,
-			gint            *y,
-			GdkModifierType *mask)
+_gdk_windowing_window_get_pointer (GdkWindow       *window,
+				   gint            *x,
+				   gint            *y,
+				   GdkModifierType *mask)
 {
   GdkWindow *return_val;
   Window root;
@@ -1968,8 +2042,8 @@ gdk_window_get_pointer (GdkWindow       *window,
 }
 
 GdkWindow*
-gdk_window_at_pointer (gint *win_x,
-		       gint *win_y)
+_gdk_windowing_window_at_pointer (gint *win_x,
+				  gint *win_y)
 {
   GdkWindow *window;
   Window root;
@@ -1982,8 +2056,8 @@ gdk_window_at_pointer (gint *win_x,
   
   xwindow = GDK_ROOT_WINDOW ();
   xdisplay = GDK_DISPLAY ();
-  
-  XGrabServer (xdisplay);
+
+  gdk_x11_grab_server ();
   while (xwindow)
     {
       xwindow_last = xwindow;
@@ -1993,7 +2067,7 @@ gdk_window_at_pointer (gint *win_x,
 		     &winx, &winy,
 		     &xmask);
     }
-  XUngrabServer (xdisplay);
+  gdk_x11_ungrab_server ();
   
   window = gdk_window_lookup (xwindow_last);
   
@@ -2251,7 +2325,6 @@ gdk_window_set_override_redirect (GdkWindow *window,
  * gdk_window_set_icon_list:
  * @window: The #GdkWindow toplevel window to set the icon of.
  * @pixbufs: A list of pixbufs, of different sizes.
- * @Returns: TRUE if the icons were set, false otherwise
  *
  * Sets a list of icons for the window. One of these will be used
  * to represent the window when it has been iconified. The icon is
@@ -2261,12 +2334,8 @@ gdk_window_set_override_redirect (GdkWindow *window,
  * image quality since the window manager may only need to scale the
  * icon by a small amount or not at all.
  *
- * On the X11 backend this call might fail if the window manager
- * doesn't support the Extended Window Manager Hints. Then this
- * function returns FALSE, and the application should fall back
- * to #gdk_window_set_icon().
  **/
-gboolean
+void
 gdk_window_set_icon_list (GdkWindow *window,
 			  GList     *pixbufs)
 {
@@ -2280,14 +2349,10 @@ gdk_window_set_icon_list (GdkWindow *window,
   gint x, y;
   gint n_channels;
   
-  g_return_val_if_fail (window != NULL, FALSE);
-  g_return_val_if_fail (GDK_IS_WINDOW (window), FALSE);
+  g_return_if_fail (GDK_IS_WINDOW (window));
 
   if (GDK_WINDOW_DESTROYED (window))
-    return FALSE;
-
-  if (!gdk_net_wm_supports (gdk_atom_intern ("_NET_WM_ICON", FALSE)))
-    return FALSE;
+    return;
   
   l = pixbufs;
   size = 0;
@@ -2344,14 +2409,21 @@ gdk_window_set_icon_list (GdkWindow *window,
       l = g_list_next (l);
     }
 
-  XChangeProperty (GDK_WINDOW_XDISPLAY (window),
-		   GDK_WINDOW_XID (window),
-		   gdk_atom_intern ("_NET_WM_ICON", FALSE),
-		   XA_CARDINAL, 32,
-		   PropModeReplace,
-		   (guchar*) data, size);
-
-  return TRUE;
+  if (size > 0)
+    {
+      XChangeProperty (GDK_WINDOW_XDISPLAY (window),
+                       GDK_WINDOW_XID (window),
+                       gdk_atom_intern ("_NET_WM_ICON", FALSE),
+                       XA_CARDINAL, 32,
+                       PropModeReplace,
+                       (guchar*) data, size);
+    }
+  else
+    {
+      XDeleteProperty (GDK_WINDOW_XDISPLAY (window),
+                       GDK_WINDOW_XID (window),
+                       gdk_atom_intern ("_NET_WM_ICON", FALSE));
+    }
 }
 
 void          
@@ -3352,10 +3424,10 @@ gdk_window_xid_at_coords (gint     x,
   root = GDK_WINDOW_XID (window);
   num = g_list_length (excludes);
   
-  XGrabServer (xdisplay);
+  gdk_x11_grab_server ();
   if (!XQueryTree (xdisplay, root, &root_win, &parent_win, &list, &num))
     {
-      XUngrabServer (xdisplay);
+      gdk_x11_ungrab_server ();
       return root;
     }
   if (list)
@@ -3381,20 +3453,20 @@ gdk_window_xid_at_coords (gint     x,
 	      if (!g_list_find (excludes, (gpointer *) child))
 		{
 		  XFree (list);
-		  XUngrabServer (xdisplay);
+                  gdk_x11_ungrab_server ();
 		  return child;
 		}
 	    }
 	  else
 	    {
 	      XFree (list);
-	      XUngrabServer (xdisplay);
+              gdk_x11_ungrab_server ();
 	      return child;
 	    }
 	} while (--i > 0);
       XFree (list);
     }
-  XUngrabServer (xdisplay);
+  gdk_x11_ungrab_server ();
   return root;
 }
 
@@ -3814,4 +3886,3 @@ gdk_window_begin_move_drag (GdkWindow *window,
   else
     emulate_move_drag (window, button, root_x, root_y, timestamp);
 }
-
