@@ -335,7 +335,7 @@ sized_icon_set_from_inline (const guchar *inline_data,
 
   set = gtk_icon_set_new ();
 
-  source.pixbuf = gdk_pixbuf_new_from_inline (inline_data, FALSE, -1, NULL);
+  source.pixbuf = gdk_pixbuf_new_from_stream (-1, inline_data, FALSE, NULL);
 
   g_assert (source.pixbuf);
 
@@ -361,7 +361,7 @@ sized_with_fallback_icon_set_from_inline (const guchar *fallback_data,
 
   set = gtk_icon_set_new ();
 
-  source.pixbuf = gdk_pixbuf_new_from_inline (inline_data, FALSE, -1, NULL);
+  source.pixbuf = gdk_pixbuf_new_from_stream (-1, inline_data, FALSE, NULL);
 
   g_assert (source.pixbuf);
 
@@ -371,7 +371,7 @@ sized_with_fallback_icon_set_from_inline (const guchar *fallback_data,
   
   source.any_size = TRUE;
 
-  source.pixbuf = gdk_pixbuf_new_from_inline (fallback_data, FALSE, -1, NULL);
+  source.pixbuf = gdk_pixbuf_new_from_stream (-1, fallback_data, FALSE, NULL);
 
   g_assert (source.pixbuf);
 
@@ -393,7 +393,7 @@ unsized_icon_set_from_inline (const guchar *inline_data)
 
   set = gtk_icon_set_new ();
 
-  source.pixbuf = gdk_pixbuf_new_from_inline (inline_data, FALSE, -1, NULL);
+  source.pixbuf = gdk_pixbuf_new_from_stream (-1, inline_data, FALSE, NULL);
 
   g_assert (source.pixbuf);
 
@@ -461,6 +461,10 @@ get_default_icons (GtkIconFactory *factory)
   add_sized (factory, dialog_question, GTK_ICON_SIZE_DIALOG, GTK_STOCK_DIALOG_QUESTION);
   add_sized (factory, dialog_warning, GTK_ICON_SIZE_DIALOG, GTK_STOCK_DIALOG_WARNING);
   
+  /* dnd size only */
+  add_sized (factory, stock_new, GTK_ICON_SIZE_DND, GTK_STOCK_DND);
+  add_sized (factory, stock_dnd_multiple, GTK_ICON_SIZE_DND, GTK_STOCK_DND_MULTIPLE);
+  
   /* Only have button sizes */
   add_sized (factory, stock_button_apply, GTK_ICON_SIZE_BUTTON, GTK_STOCK_APPLY);
   add_sized (factory, stock_button_cancel, GTK_ICON_SIZE_BUTTON, GTK_STOCK_CANCEL);
@@ -585,7 +589,7 @@ init_icon_sizes (void)
 {
   if (icon_sizes == NULL)
     {
-#define NUM_BUILTIN_SIZES 6
+#define NUM_BUILTIN_SIZES 7
       gint i;
 
       icon_aliases = g_hash_table_new (g_str_hash, g_str_equal);
@@ -625,6 +629,11 @@ init_icon_sizes (void)
       icon_sizes[GTK_ICON_SIZE_LARGE_TOOLBAR].width = 24;
       icon_sizes[GTK_ICON_SIZE_LARGE_TOOLBAR].height = 24;
 
+      icon_sizes[GTK_ICON_SIZE_DND].size = GTK_ICON_SIZE_DND;
+      icon_sizes[GTK_ICON_SIZE_DND].name = "gtk-dnd";
+      icon_sizes[GTK_ICON_SIZE_DND].width = 32;
+      icon_sizes[GTK_ICON_SIZE_DND].height = 32;
+
       icon_sizes[GTK_ICON_SIZE_DIALOG].size = GTK_ICON_SIZE_DIALOG;
       icon_sizes[GTK_ICON_SIZE_DIALOG].name = "gtk-dialog";
       icon_sizes[GTK_ICON_SIZE_DIALOG].width = 48;
@@ -718,12 +727,11 @@ gtk_icon_size_register (const gchar *name,
   icon_sizes[icon_sizes_used].width = width;
   icon_sizes[icon_sizes_used].height = height;
 
-  /* alias to self. */
-  gtk_icon_size_register_alias (icon_sizes[icon_sizes_used].name,
-                                icon_sizes[icon_sizes_used].size);
-  
   ++icon_sizes_used;
 
+  /* alias to self. */
+  gtk_icon_size_register_alias (name, icon_sizes_used - 1);
+  
   return icon_sizes_used - 1;
 }
 
@@ -1028,21 +1036,11 @@ find_and_prep_icon_source (GtkIconSet       *icon_set,
   
   if (source->pixbuf == NULL)
     {
-      GError *error;
-      gchar *full;
+      GError *error = NULL;
       
       g_assert (source->filename);
+      source->pixbuf = gdk_pixbuf_new_from_file (source->filename, &error);
 
-      if (g_path_is_absolute (source->filename))
-        full = g_strdup (source->filename);
-      else
-        full = gtk_rc_find_pixmap_in_path (NULL, source->filename);
-
-      error = NULL;
-      source->pixbuf = gdk_pixbuf_new_from_file (full, &error);
-
-      g_free (full);
-      
       if (source->pixbuf == NULL)
         {
           /* Remove this icon source so we don't keep trying to
@@ -1081,7 +1079,7 @@ render_fallback_image (GtkStyle          *style,
   static GtkIconSource fallback_source = { NULL, NULL, 0, 0, 0, TRUE, TRUE, TRUE };
 
   if (fallback_source.pixbuf == NULL)
-    fallback_source.pixbuf = gdk_pixbuf_new_from_inline (MISSING_IMAGE_INLINE, FALSE, -1, NULL);
+    fallback_source.pixbuf = gdk_pixbuf_new_from_stream (-1, MISSING_IMAGE_INLINE, FALSE, NULL);
   
   return gtk_style_render_icon (style,
                                 &fallback_source,
@@ -1426,19 +1424,15 @@ gtk_icon_source_free (GtkIconSource *source)
  * @source: a #GtkIconSource
  * @filename: image file to use
  *
- * Sets the name of an image file to use as a base image when creating icon
- * variants for #GtkIconSet. If the filename is absolute, GTK+ will
- * attempt to open the exact file given. If the filename is relative,
- * GTK+ will search for it in the "pixmap path" which can be configured
- * by users in their gtkrc files or specified as part of a theme's gtkrc
- * file. See #GtkRcStyle for information on gtkrc files.
- * 
+ * Sets the name of an image file to use as a base image when creating
+ * icon variants for #GtkIconSet. The filename must be absolute. 
  **/
 void
-gtk_icon_source_set_filename             (GtkIconSource *source,
-                                          const gchar   *filename)
+gtk_icon_source_set_filename (GtkIconSource *source,
+			      const gchar   *filename)
 {
   g_return_if_fail (source != NULL);
+  g_return_if_fail (filename == NULL || g_path_is_absolute (filename));
 
   if (source->filename == filename)
     return;
