@@ -197,7 +197,6 @@ gtk_menu_item_init (GtkMenuItem *menu_item)
   menu_item->accelerator_signal = menu_item_signals[ACTIVATE];
   menu_item->toggle_size = 0;
   menu_item->accelerator_width = 0;
-  menu_item->show_toggle_indicator = FALSE;
   menu_item->show_submenu_indicator = FALSE;
   menu_item->submenu_direction = GTK_DIRECTION_RIGHT;
   menu_item->submenu_placement = GTK_TOP_BOTTOM;
@@ -263,7 +262,6 @@ gtk_menu_item_destroy (GtkObject *object)
 {
   GtkMenuItem *menu_item;
 
-  g_return_if_fail (object != NULL);
   g_return_if_fail (GTK_IS_MENU_ITEM (object));
 
   menu_item = GTK_MENU_ITEM (object);
@@ -281,7 +279,6 @@ gtk_menu_item_detacher (GtkWidget     *widget,
 {
   GtkMenuItem *menu_item;
 
-  g_return_if_fail (widget != NULL);
   g_return_if_fail (GTK_IS_MENU_ITEM (widget));
 
   menu_item = GTK_MENU_ITEM (widget);
@@ -294,7 +291,6 @@ void
 gtk_menu_item_set_submenu (GtkMenuItem *menu_item,
 			   GtkWidget   *submenu)
 {
-  g_return_if_fail (menu_item != NULL);
   g_return_if_fail (GTK_IS_MENU_ITEM (menu_item));
   
   if (menu_item->submenu != submenu)
@@ -311,10 +307,26 @@ gtk_menu_item_set_submenu (GtkMenuItem *menu_item,
     }
 }
 
+/**
+ * gtk_menu_item_get_submenu:
+ * @menu_item: a #GtkMenuItem
+ *
+ * Gets the submenu underneath this menu item, if any. See
+ * gtk_menu_item_set_submenu().
+ *
+ * Return value: submenu for this menu item, or %NULL if none.
+ **/
+GtkWidget *
+gtk_menu_item_get_submenu (GtkMenuItem *menu_item)
+{
+  g_return_val_if_fail (GTK_IS_MENU_ITEM (menu_item), NULL);
+
+  return menu_item->submenu;
+}
+
 void
 gtk_menu_item_remove_submenu (GtkMenuItem         *menu_item)
 {
-  g_return_if_fail (menu_item != NULL);
   g_return_if_fail (GTK_IS_MENU_ITEM (menu_item));
       
   if (menu_item->submenu)
@@ -325,28 +337,14 @@ void
 gtk_menu_item_set_placement (GtkMenuItem         *menu_item,
 			     GtkSubmenuPlacement  placement)
 {
-  g_return_if_fail (menu_item != NULL);
   g_return_if_fail (GTK_IS_MENU_ITEM (menu_item));
 
   menu_item->submenu_placement = placement;
 }
 
-void
-gtk_menu_item_configure (GtkMenuItem *menu_item,
-			 gboolean     show_toggle_indicator,
-			 gboolean     show_submenu_indicator)
-{
-  g_return_if_fail (menu_item != NULL);
-  g_return_if_fail (GTK_IS_MENU_ITEM (menu_item));
-
-  menu_item->show_toggle_indicator = show_toggle_indicator;
-  menu_item->show_submenu_indicator = show_submenu_indicator;
-}
-
 void
 gtk_menu_item_select (GtkMenuItem *menu_item)
 {
-  g_return_if_fail (menu_item != NULL);
   g_return_if_fail (GTK_IS_MENU_ITEM (menu_item));
   
   gtk_item_select (GTK_ITEM (menu_item));
@@ -355,7 +353,6 @@ gtk_menu_item_select (GtkMenuItem *menu_item)
 void
 gtk_menu_item_deselect (GtkMenuItem *menu_item)
 {
-  g_return_if_fail (menu_item != NULL);
   g_return_if_fail (GTK_IS_MENU_ITEM (menu_item));
   
   gtk_item_deselect (GTK_ITEM (menu_item));
@@ -364,7 +361,6 @@ gtk_menu_item_deselect (GtkMenuItem *menu_item)
 void
 gtk_menu_item_activate (GtkMenuItem *menu_item)
 {
-  g_return_if_fail (menu_item != NULL);
   g_return_if_fail (GTK_IS_MENU_ITEM (menu_item));
   
   gtk_signal_emit (GTK_OBJECT (menu_item), menu_item_signals[ACTIVATE]);
@@ -374,7 +370,6 @@ void
 gtk_menu_item_toggle_size_request (GtkMenuItem *menu_item,
 				   gint        *requisition)
 {
-  g_return_if_fail (menu_item != NULL);
   g_return_if_fail (GTK_IS_MENU_ITEM (menu_item));
 
   gtk_signal_emit (GTK_OBJECT (menu_item), menu_item_signals[TOGGLE_SIZE_REQUEST], requisition);
@@ -384,7 +379,6 @@ void
 gtk_menu_item_toggle_size_allocate (GtkMenuItem *menu_item,
 				    gint         allocation)
 {
-  g_return_if_fail (menu_item != NULL);
   g_return_if_fail (GTK_IS_MENU_ITEM (menu_item));
 
   gtk_signal_emit (GTK_OBJECT (menu_item), menu_item_signals[TOGGLE_SIZE_ALLOCATE], allocation);
@@ -417,7 +411,6 @@ gtk_menu_item_size_request (GtkWidget      *widget,
   GtkBin *bin;
   guint accel_width;
 
-  g_return_if_fail (widget != NULL);
   g_return_if_fail (GTK_IS_MENU_ITEM (widget));
   g_return_if_fail (requisition != NULL);
 
@@ -458,7 +451,6 @@ gtk_menu_item_size_allocate (GtkWidget     *widget,
   GtkBin *bin;
   GtkAllocation child_allocation;
 
-  g_return_if_fail (widget != NULL);
   g_return_if_fail (GTK_IS_MENU_ITEM (widget));
   g_return_if_fail (allocation != NULL);
 
@@ -503,7 +495,6 @@ gtk_menu_item_paint (GtkWidget    *widget,
   gint width, height;
   gint x, y;
 
-  g_return_if_fail (widget != NULL);
   g_return_if_fail (GTK_IS_MENU_ITEM (widget));
 
   if (GTK_WIDGET_DRAWABLE (widget))
@@ -556,7 +547,6 @@ static gint
 gtk_menu_item_expose (GtkWidget      *widget,
 		      GdkEventExpose *event)
 {
-  g_return_val_if_fail (widget != NULL, FALSE);
   g_return_val_if_fail (GTK_IS_MENU_ITEM (widget), FALSE);
   g_return_val_if_fail (event != NULL, FALSE);
 
@@ -575,7 +565,6 @@ gtk_real_menu_item_select (GtkItem *item)
 {
   GtkMenuItem *menu_item;
 
-  g_return_if_fail (item != NULL);
   g_return_if_fail (GTK_IS_MENU_ITEM (item));
 
   menu_item = GTK_MENU_ITEM (item);
@@ -594,7 +583,8 @@ gtk_real_menu_item_select (GtkItem *item)
 					    menu_item);
       else
 	gtk_menu_item_popup_submenu (menu_item);
-      if(event) gdk_event_free(event);
+      if (event)
+	gdk_event_free(event);
     }
   
   gtk_widget_set_state (GTK_WIDGET (menu_item), GTK_STATE_PRELIGHT);
@@ -606,7 +596,6 @@ gtk_real_menu_item_deselect (GtkItem *item)
 {
   GtkMenuItem *menu_item;
 
-  g_return_if_fail (item != NULL);
   g_return_if_fail (GTK_IS_MENU_ITEM (item));
 
   menu_item = GTK_MENU_ITEM (item);
@@ -627,7 +616,8 @@ gtk_real_menu_item_deselect (GtkItem *item)
       etime = event ? gdk_event_get_time (event) : GDK_CURRENT_TIME;
       if (etime > last_submenu_deselect_time)
 	last_submenu_deselect_time = etime;
-      if(event) gdk_event_free(event);
+      if (event)
+	gdk_event_free(event);
     }
 
   gtk_widget_set_state (GTK_WIDGET (menu_item), GTK_STATE_NORMAL);
@@ -659,7 +649,6 @@ gtk_real_menu_item_activate_item (GtkMenuItem *menu_item)
   GtkWidget *widget;
   GtkMenuShell *submenu; 
 
-  g_return_if_fail (menu_item != NULL);
   g_return_if_fail (GTK_IS_MENU_ITEM (menu_item));
 
   widget = GTK_WIDGET (menu_item);
@@ -694,7 +683,6 @@ static void
 gtk_real_menu_item_toggle_size_request (GtkMenuItem *menu_item,
 					gint        *requisition)
 {
-  g_return_if_fail (menu_item != NULL);
   g_return_if_fail (GTK_IS_MENU_ITEM (menu_item));
 
   *requisition = 0;
@@ -704,7 +692,6 @@ static void
 gtk_real_menu_item_toggle_size_allocate (GtkMenuItem *menu_item,
 					 gint         allocation)
 {
-  g_return_if_fail (menu_item != NULL);
   g_return_if_fail (GTK_IS_MENU_ITEM (menu_item));
 
   menu_item->toggle_size = allocation;
@@ -830,13 +817,49 @@ gtk_menu_item_position_menu (GtkMenu  *menu,
   *y = ty;
 }
 
+/**
+ * gtk_menu_item_set_right_justified:
+ * @menu_item: a #GtkMenuItem.
+ * @right_justified: if %TRUE the menu item will appear at the 
+ *   far right if added to a menu bar.
+ * 
+ * Sets whether the menu item appears justified at the right
+ * side of a menu bar. This was traditionally done for "Help" menu
+ * items, but is now considered a bad idea. (If the widget
+ * layout is reversed for a right-to-left language like Hebrew
+ * or Arabic, right-justified-menu-items appear at the left.)
+ **/
 void
-gtk_menu_item_right_justify(GtkMenuItem *menuitem)
+gtk_menu_item_set_right_justified (GtkMenuItem *menu_item,
+				   gboolean     right_justified)
 {
-  g_return_if_fail (menuitem != NULL);
-  g_return_if_fail (GTK_IS_MENU_ITEM (menuitem));
+  g_return_if_fail (GTK_IS_MENU_ITEM (menu_item));
 
-  menuitem->right_justify = 1;
+  right_justified = right_justified != FALSE;
+
+  if (right_justified != menu_item->right_justify)
+    {
+      menu_item->right_justify = right_justified;
+      gtk_widget_queue_resize (GTK_WIDGET (menu_item));
+    }
+}
+
+/**
+ * gtk_menu_item_get_right_justified:
+ * @menu_item: a #GtkMenuItem
+ * 
+ * Gets whether the menu item appears justified at the right
+ * side of the menu bar.
+ * 
+ * Return value: %TRUE if the menu item will appear at the
+ *   far right if added to a menu bar.
+ **/
+gboolean
+gtk_menu_item_get_right_justified (GtkMenuItem *menu_item)
+{
+  g_return_val_if_fail (GTK_IS_MENU_ITEM (menu_item), FALSE);
+  
+  return menu_item->right_justify;
 }
 
 
@@ -845,7 +868,6 @@ gtk_menu_item_show_all (GtkWidget *widget)
 {
   GtkMenuItem *menu_item;
 
-  g_return_if_fail (widget != NULL);
   g_return_if_fail (GTK_IS_MENU_ITEM (widget));
 
   menu_item = GTK_MENU_ITEM (widget);
@@ -863,7 +885,6 @@ gtk_menu_item_hide_all (GtkWidget *widget)
 {
   GtkMenuItem *menu_item;
 
-  g_return_if_fail (widget != NULL);
   g_return_if_fail (GTK_IS_MENU_ITEM (widget));
 
   gtk_widget_hide (widget);
@@ -885,7 +906,6 @@ gtk_menu_item_forall (GtkContainer *container,
   GtkBin *bin;
   GtkMenuItem *menu_item;
 
-  g_return_if_fail (container != NULL);
   g_return_if_fail (GTK_IS_MENU_ITEM (container));
   g_return_if_fail (callback != NULL);
 
