@@ -67,7 +67,7 @@ static gint gtk_option_menu_button_press    (GtkWidget          *widget,
 					     GdkEventButton     *event);
 static gint gtk_option_menu_key_press	    (GtkWidget          *widget,
 					     GdkEventKey        *event);
-static void gtk_option_menu_deactivate      (GtkMenuShell       *menu_shell,
+static void gtk_option_menu_selection_done  (GtkMenuShell       *menu_shell,
 					     GtkOptionMenu      *option_menu);
 static void gtk_option_menu_update_contents (GtkOptionMenu      *option_menu);
 static void gtk_option_menu_remove_contents (GtkOptionMenu      *option_menu);
@@ -132,13 +132,13 @@ gtk_option_menu_class_init (GtkOptionMenuClass *class)
   parent_class = gtk_type_class (gtk_button_get_type ());
 
   signals[CHANGED] =
-    g_signal_newc ("changed",
-                   G_OBJECT_CLASS_TYPE (class),
-                   G_SIGNAL_RUN_LAST,
-                   G_STRUCT_OFFSET (GtkOptionMenuClass, changed),
-                   NULL, NULL,
-                   gtk_marshal_VOID__VOID,
-                   G_TYPE_NONE, 0);
+    g_signal_new ("changed",
+                  G_OBJECT_CLASS_TYPE (class),
+                  G_SIGNAL_RUN_LAST,
+                  G_STRUCT_OFFSET (GtkOptionMenuClass, changed),
+                  NULL, NULL,
+                  gtk_marshal_VOID__VOID,
+                  G_TYPE_NONE, 0);
   
   object_class->destroy = gtk_option_menu_destroy;
   
@@ -193,7 +193,6 @@ gtk_option_menu_new (void)
 GtkWidget*
 gtk_option_menu_get_menu (GtkOptionMenu *option_menu)
 {
-  g_return_val_if_fail (option_menu != NULL, NULL);
   g_return_val_if_fail (GTK_IS_OPTION_MENU (option_menu), NULL);
 
   return option_menu->menu;
@@ -205,7 +204,6 @@ gtk_option_menu_detacher (GtkWidget     *widget,
 {
   GtkOptionMenu *option_menu;
 
-  g_return_if_fail (widget != NULL);
   g_return_if_fail (GTK_IS_OPTION_MENU (widget));
 
   option_menu = GTK_OPTION_MENU (widget);
@@ -222,9 +220,7 @@ void
 gtk_option_menu_set_menu (GtkOptionMenu *option_menu,
 			  GtkWidget     *menu)
 {
-  g_return_if_fail (option_menu != NULL);
   g_return_if_fail (GTK_IS_OPTION_MENU (option_menu));
-  g_return_if_fail (menu != NULL);
   g_return_if_fail (GTK_IS_MENU (menu));
 
   if (option_menu->menu != menu)
@@ -238,9 +234,9 @@ gtk_option_menu_set_menu (GtkOptionMenu *option_menu,
 
       gtk_option_menu_calc_size (option_menu);
 
-      gtk_signal_connect (GTK_OBJECT (option_menu->menu), "deactivate",
-			  (GtkSignalFunc) gtk_option_menu_deactivate,
-			  option_menu);
+      gtk_signal_connect_after (GTK_OBJECT (option_menu->menu), "selection_done",
+				G_CALLBACK (gtk_option_menu_selection_done),
+				option_menu);
       gtk_signal_connect_object (GTK_OBJECT (option_menu->menu), "size_request",
 				 (GtkSignalFunc) gtk_option_menu_calc_size,
 				 GTK_OBJECT (option_menu));
@@ -255,7 +251,6 @@ gtk_option_menu_set_menu (GtkOptionMenu *option_menu,
 void
 gtk_option_menu_remove_menu (GtkOptionMenu *option_menu)
 {
-  g_return_if_fail (option_menu != NULL);
   g_return_if_fail (GTK_IS_OPTION_MENU (option_menu));
 
   if (option_menu->menu)
@@ -268,7 +263,6 @@ gtk_option_menu_set_history (GtkOptionMenu *option_menu,
 {
   GtkWidget *menu_item;
 
-  g_return_if_fail (option_menu != NULL);
   g_return_if_fail (GTK_IS_OPTION_MENU (option_menu));
 
   if (option_menu->menu)
@@ -316,7 +310,6 @@ gtk_option_menu_destroy (GtkObject *object)
 {
   GtkOptionMenu *option_menu;
 
-  g_return_if_fail (object != NULL);
   g_return_if_fail (GTK_IS_OPTION_MENU (object));
 
   option_menu = GTK_OPTION_MENU (object);
@@ -433,7 +426,6 @@ gtk_option_menu_paint (GtkWidget    *widget,
   GtkOptionMenuProps props;
   gint border_width;
 
-  g_return_if_fail (widget != NULL);
   g_return_if_fail (GTK_IS_OPTION_MENU (widget));
   g_return_if_fail (area != NULL);
 
@@ -462,11 +454,11 @@ gtk_option_menu_paint (GtkWidget    *widget,
 	  gdk_window_clear_area (widget->window, area->x, area->y, area->width, area->height);
 	}
 
-      gtk_paint_box(widget->style, widget->window,
-		    GTK_WIDGET_STATE (widget), GTK_SHADOW_OUT,
-		    area, widget, "optionmenu",
-		    button_area.x, button_area.y,
-		    button_area.width, button_area.height);
+      gtk_paint_box (widget->style, widget->window,
+		     GTK_WIDGET_STATE (widget), GTK_SHADOW_OUT,
+		     area, widget, "optionmenu",
+		     button_area.x, button_area.y,
+		     button_area.width, button_area.height);
       
       gtk_paint_tab (widget->style, widget->window,
 		     GTK_WIDGET_STATE (widget), GTK_SHADOW_OUT,
@@ -509,7 +501,6 @@ static gint
 gtk_option_menu_expose (GtkWidget      *widget,
 			GdkEventExpose *event)
 {
-  g_return_val_if_fail (widget != NULL, FALSE);
   g_return_val_if_fail (GTK_IS_OPTION_MENU (widget), FALSE);
   g_return_val_if_fail (event != NULL, FALSE);
 
@@ -569,7 +560,6 @@ gtk_option_menu_button_press (GtkWidget      *widget,
   GtkOptionMenu *option_menu;
   GtkWidget *menu_item;
 
-  g_return_val_if_fail (widget != NULL, FALSE);
   g_return_val_if_fail (GTK_IS_OPTION_MENU (widget), FALSE);
   g_return_val_if_fail (event != NULL, FALSE);
 
@@ -598,7 +588,6 @@ gtk_option_menu_key_press (GtkWidget   *widget,
   GtkOptionMenu *option_menu;
   GtkWidget *menu_item;
 
-  g_return_val_if_fail (widget != NULL, FALSE);
   g_return_val_if_fail (GTK_IS_OPTION_MENU (widget), FALSE);
   g_return_val_if_fail (event != NULL, FALSE);
 
@@ -606,6 +595,7 @@ gtk_option_menu_key_press (GtkWidget   *widget,
 
   switch (event->keyval)
     {
+    case GDK_KP_Space:
     case GDK_space:
       gtk_option_menu_remove_contents (option_menu);
       gtk_menu_popup (GTK_MENU (option_menu->menu), NULL, NULL,
@@ -621,11 +611,10 @@ gtk_option_menu_key_press (GtkWidget   *widget,
 }
 
 static void
-gtk_option_menu_deactivate (GtkMenuShell  *menu_shell,
-			    GtkOptionMenu *option_menu)
+gtk_option_menu_selection_done (GtkMenuShell  *menu_shell,
+				GtkOptionMenu *option_menu)
 {
   g_return_if_fail (menu_shell != NULL);
-  g_return_if_fail (option_menu != NULL);
   g_return_if_fail (GTK_IS_OPTION_MENU (option_menu));
 
   gtk_option_menu_update_contents (option_menu);
@@ -695,7 +684,6 @@ gtk_option_menu_update_contents (GtkOptionMenu *option_menu)
   GtkWidget *child;
   GtkRequisition child_requisition;
 
-  g_return_if_fail (option_menu != NULL);
   g_return_if_fail (GTK_IS_OPTION_MENU (option_menu));
 
   if (option_menu->menu)
@@ -739,7 +727,6 @@ gtk_option_menu_remove_contents (GtkOptionMenu *option_menu)
 {
   GtkWidget *child;
   
-  g_return_if_fail (option_menu != NULL);
   g_return_if_fail (GTK_IS_OPTION_MENU (option_menu));
 
   if (option_menu->menu_item)
@@ -773,7 +760,6 @@ gtk_option_menu_calc_size (GtkOptionMenu *option_menu)
   gint old_width = option_menu->width;
   gint old_height = option_menu->height;
 
-  g_return_if_fail (option_menu != NULL);
   g_return_if_fail (GTK_IS_OPTION_MENU (option_menu));
 
   option_menu->width = 0;
@@ -818,7 +804,6 @@ gtk_option_menu_position (GtkMenu  *menu,
   gint menu_ypos;
   gint menu_width;
 
-  g_return_if_fail (user_data != NULL);
   g_return_if_fail (GTK_IS_OPTION_MENU (user_data));
 
   option_menu = GTK_OPTION_MENU (user_data);
@@ -873,7 +858,6 @@ gtk_option_menu_show_all (GtkWidget *widget)
   GtkContainer *container;
   GtkOptionMenu *option_menu;
   
-  g_return_if_fail (widget != NULL);
   g_return_if_fail (GTK_IS_OPTION_MENU (widget));
   container = GTK_CONTAINER (widget);
   option_menu = GTK_OPTION_MENU (widget);
@@ -892,7 +876,6 @@ gtk_option_menu_hide_all (GtkWidget *widget)
 {
   GtkContainer *container;
 
-  g_return_if_fail (widget != NULL);
   g_return_if_fail (GTK_IS_OPTION_MENU (widget));
   container = GTK_CONTAINER (widget);
 
