@@ -43,9 +43,9 @@
 #define ARROW_SIZE                         11
 #define SPIN_BUTTON_INITIAL_TIMER_DELAY    200
 #define SPIN_BUTTON_TIMER_DELAY            20
-#define MAX_TEXT_LENGTH                    256
 #define MAX_TIMER_CALLS                    5
 #define EPSILON                            1e-5
+#define	MAX_DIGITS			   20
 
 enum {
   PROP_0,
@@ -221,7 +221,7 @@ gtk_spin_button_class_init (GtkSpinButtonClass *class)
 						      _("Digits"),
 						      _("The number of decimal places to display"),
 						      0,
-						      5,
+						      MAX_DIGITS,
 						      0,
 						      G_PARAM_READWRITE));
   
@@ -283,13 +283,13 @@ gtk_spin_button_class_init (GtkSpinButtonClass *class)
 		    GTK_TYPE_INT, 1, GTK_TYPE_POINTER);
 
   spinbutton_signals[OUTPUT] =
-    g_signal_newc ("output",
-		   G_TYPE_FROM_CLASS(object_class),
-		   G_SIGNAL_RUN_LAST,
-		   G_STRUCT_OFFSET(GtkSpinButtonClass, output),
-		   _gtk_boolean_handled_accumulator, NULL,
-		   gtk_marshal_BOOLEAN__VOID,
-		   G_TYPE_BOOLEAN, 0);
+    g_signal_new ("output",
+                  G_TYPE_FROM_CLASS(object_class),
+                  G_SIGNAL_RUN_LAST,
+                  G_STRUCT_OFFSET(GtkSpinButtonClass, output),
+                  _gtk_boolean_handled_accumulator, NULL,
+                  gtk_marshal_BOOLEAN__VOID,
+                  G_TYPE_BOOLEAN, 0);
 
   spinbutton_signals[VALUE_CHANGED] =
     gtk_signal_new ("value_changed",
@@ -430,7 +430,6 @@ gtk_spin_button_finalize (GObject *object)
 static void
 gtk_spin_button_map (GtkWidget *widget)
 {
-  g_return_if_fail (widget != NULL);
   g_return_if_fail (GTK_IS_SPIN_BUTTON (widget));
 
   if (GTK_WIDGET_REALIZED (widget) && !GTK_WIDGET_MAPPED (widget))
@@ -443,7 +442,6 @@ gtk_spin_button_map (GtkWidget *widget)
 static void
 gtk_spin_button_unmap (GtkWidget *widget)
 {
-  g_return_if_fail (widget != NULL);
   g_return_if_fail (GTK_IS_SPIN_BUTTON (widget));
 
   if (GTK_WIDGET_MAPPED (widget))
@@ -462,7 +460,6 @@ gtk_spin_button_realize (GtkWidget *widget)
   guint real_width;
   gint return_val;
 
-  g_return_if_fail (widget != NULL);
   g_return_if_fail (GTK_IS_SPIN_BUTTON (widget));
   
   spin_button = GTK_SPIN_BUTTON (widget);
@@ -511,7 +508,6 @@ gtk_spin_button_unrealize (GtkWidget *widget)
 {
   GtkSpinButton *spin;
 
-  g_return_if_fail (widget != NULL);
   g_return_if_fail (GTK_IS_SPIN_BUTTON (widget));
 
   spin = GTK_SPIN_BUTTON (widget);
@@ -556,7 +552,6 @@ gtk_spin_button_size_request (GtkWidget      *widget,
   GtkEntry *entry;
   GtkSpinButton *spin_button;
   
-  g_return_if_fail (widget != NULL);
   g_return_if_fail (requisition != NULL);
   g_return_if_fail (GTK_IS_SPIN_BUTTON (widget));
 
@@ -567,20 +562,17 @@ gtk_spin_button_size_request (GtkWidget      *widget,
 
   if (entry->width_chars < 0)
     {
+      PangoContext *context;
       PangoFontMetrics metrics;
-      PangoFont *font;
-      gchar *lang;
       gint width;
       gint w;
       int string_len;
 
-
-      font = pango_context_load_font (gtk_widget_get_pango_context (widget),
-                                      widget->style->font_desc);
-      lang = pango_context_get_lang (gtk_widget_get_pango_context (widget));
-      pango_font_get_metrics (font, lang, &metrics);
-      g_free (lang);
-      g_object_unref (G_OBJECT (font));
+      context = gtk_widget_get_pango_context (widget);
+      pango_context_get_metrics (context,
+				 widget->style->font_desc,
+				 pango_context_get_language (context),
+				 &metrics);
       
       /* Get max of MIN_SPIN_BUTTON_WIDTH, size of upper, size of lower */
       
@@ -607,7 +599,6 @@ gtk_spin_button_size_allocate (GtkWidget     *widget,
 {
   GtkAllocation child_allocation;
 
-  g_return_if_fail (widget != NULL);
   g_return_if_fail (GTK_IS_SPIN_BUTTON (widget));
   g_return_if_fail (allocation != NULL);
 
@@ -649,7 +640,6 @@ gtk_spin_button_expose (GtkWidget      *widget,
 {
   GtkSpinButton *spin;
 
-  g_return_val_if_fail (widget != NULL, FALSE);
   g_return_val_if_fail (GTK_IS_SPIN_BUTTON (widget), FALSE);
   g_return_val_if_fail (event != NULL, FALSE);
 
@@ -699,7 +689,6 @@ gtk_spin_button_draw_arrow (GtkSpinButton *spin_button,
   gint x;
   gint y;
 
-  g_return_if_fail (spin_button != NULL);
   g_return_if_fail (GTK_IS_SPIN_BUTTON (spin_button));
   
   widget = GTK_WIDGET (spin_button);
@@ -782,7 +771,6 @@ gtk_spin_button_enter_notify (GtkWidget        *widget,
 {
   GtkSpinButton *spin;
 
-  g_return_val_if_fail (widget != NULL, FALSE);
   g_return_val_if_fail (GTK_IS_SPIN_BUTTON (widget), FALSE);
   g_return_val_if_fail (event != NULL, FALSE);
 
@@ -817,7 +805,6 @@ gtk_spin_button_leave_notify (GtkWidget        *widget,
 {
   GtkSpinButton *spin;
 
-  g_return_val_if_fail (widget != NULL, FALSE);
   g_return_val_if_fail (GTK_IS_SPIN_BUTTON (widget), FALSE);
   g_return_val_if_fail (event != NULL, FALSE);
 
@@ -843,7 +830,6 @@ static gint
 gtk_spin_button_focus_out (GtkWidget     *widget,
 			   GdkEventFocus *event)
 {
-  g_return_val_if_fail (widget != NULL, FALSE);
   g_return_val_if_fail (GTK_IS_SPIN_BUTTON (widget), FALSE);
   g_return_val_if_fail (event != NULL, FALSE);
 
@@ -859,7 +845,6 @@ gtk_spin_button_scroll (GtkWidget      *widget,
 {
   GtkSpinButton *spin;
 
-  g_return_val_if_fail (widget != NULL, FALSE);
   g_return_val_if_fail (GTK_IS_SPIN_BUTTON (widget), FALSE);
   g_return_val_if_fail (event != NULL, FALSE);
 
@@ -889,7 +874,6 @@ gtk_spin_button_button_press (GtkWidget      *widget,
 {
   GtkSpinButton *spin;
 
-  g_return_val_if_fail (widget != NULL, FALSE);
   g_return_val_if_fail (GTK_IS_SPIN_BUTTON (widget), FALSE);
   g_return_val_if_fail (event != NULL, FALSE);
 
@@ -983,7 +967,6 @@ gtk_spin_button_button_release (GtkWidget      *widget,
 {
   GtkSpinButton *spin;
 
-  g_return_val_if_fail (widget != NULL, FALSE);
   g_return_val_if_fail (GTK_IS_SPIN_BUTTON (widget), FALSE);
   g_return_val_if_fail (event != NULL, FALSE);
 
@@ -1046,7 +1029,6 @@ gtk_spin_button_motion_notify (GtkWidget      *widget,
 {
   GtkSpinButton *spin;
 
-  g_return_val_if_fail (widget != NULL, FALSE);
   g_return_val_if_fail (GTK_IS_SPIN_BUTTON (widget), FALSE);
   g_return_val_if_fail (event != NULL, FALSE);
 
@@ -1132,7 +1114,6 @@ gtk_spin_button_value_changed (GtkAdjustment *adjustment,
 {
   gint return_val;
 
-  g_return_if_fail (adjustment != NULL);
   g_return_if_fail (GTK_IS_ADJUSTMENT (adjustment));
 
   return_val = FALSE;
@@ -1157,8 +1138,8 @@ gtk_spin_button_key_press (GtkWidget     *widget,
   GtkSpinButton *spin;
   gint key;
   gboolean key_repeat = FALSE;
-
-  g_return_val_if_fail (widget != NULL, FALSE);
+  gboolean retval = FALSE;
+  
   g_return_val_if_fail (GTK_IS_SPIN_BUTTON (widget), FALSE);
   g_return_val_if_fail (event != NULL, FALSE);
   
@@ -1167,100 +1148,112 @@ gtk_spin_button_key_press (GtkWidget     *widget,
 
   key_repeat = (event->time == spin->ev_time);
 
-  if (GTK_ENTRY (widget)->editable &&
-      (key == GDK_Up || key == GDK_Down || 
-       key == GDK_Page_Up || key == GDK_Page_Down))
-    gtk_spin_button_update (spin);
-
-  switch (key)
+  if (GTK_ENTRY (widget)->editable)
     {
-    case GDK_Up:
+      switch (key)
+        {
+        case GDK_KP_Up:
+        case GDK_Up:
 
-      if (GTK_WIDGET_HAS_FOCUS (widget))
-	{
-	  gtk_signal_emit_stop_by_name (GTK_OBJECT (widget), 
-					"key_press_event");
-	  if (!key_repeat)
-	    spin->timer_step = spin->adjustment->step_increment;
+          if (GTK_WIDGET_HAS_FOCUS (widget))
+            {
+              gtk_signal_emit_stop_by_name (GTK_OBJECT (widget), 
+                                            "key_press_event");
+              if (!key_repeat)
+                spin->timer_step = spin->adjustment->step_increment;
 
-	 gtk_spin_button_real_spin (spin, spin->timer_step);
+              gtk_spin_button_real_spin (spin, spin->timer_step);
 
-	  if (key_repeat)
-	    {
-	      if (spin->climb_rate > 0.0 && spin->timer_step
-		  < spin->adjustment->page_increment)
-		{
-		  if (spin->timer_calls < MAX_TIMER_CALLS)
-		    spin->timer_calls++;
-		  else 
-		    {
-		      spin->timer_calls = 0;
-		      spin->timer_step += spin->climb_rate;
-		    }
-		}
-	    }
-	  return TRUE;
-	}
-      return FALSE;
+              if (key_repeat)
+                {
+                  if (spin->climb_rate > 0.0 && spin->timer_step
+                      < spin->adjustment->page_increment)
+                    {
+                      if (spin->timer_calls < MAX_TIMER_CALLS)
+                        spin->timer_calls++;
+                      else 
+                        {
+                          spin->timer_calls = 0;
+                          spin->timer_step += spin->climb_rate;
+                        }
+                    }
+                }
+              retval = TRUE;
+            }
+          break;
 
-    case GDK_Down:
+        case GDK_KP_Down:
+        case GDK_Down:
 
-      if (GTK_WIDGET_HAS_FOCUS (widget))
-	{
-	  gtk_signal_emit_stop_by_name (GTK_OBJECT (widget), 
-					"key_press_event");
-	  if (!key_repeat)
-	    spin->timer_step = spin->adjustment->step_increment;
+          if (GTK_WIDGET_HAS_FOCUS (widget))
+            {
+              gtk_signal_emit_stop_by_name (GTK_OBJECT (widget), 
+                                            "key_press_event");
+              if (!key_repeat)
+                spin->timer_step = spin->adjustment->step_increment;
 
-	 gtk_spin_button_real_spin (spin, -spin->timer_step);
+              gtk_spin_button_real_spin (spin, -spin->timer_step);
 
-	  if (key_repeat)
-	    {
-	      if (spin->climb_rate > 0.0 && spin->timer_step
-		  < spin->adjustment->page_increment)
-		{
-		  if (spin->timer_calls < MAX_TIMER_CALLS)
-		    spin->timer_calls++;
-		  else 
-		    {
-		      spin->timer_calls = 0;
-		      spin->timer_step += spin->climb_rate;
-		    }
-		}
-	    }
-	  return TRUE;
-	}
-      return FALSE;
+              if (key_repeat)
+                {
+                  if (spin->climb_rate > 0.0 && spin->timer_step
+                      < spin->adjustment->page_increment)
+                    {
+                      if (spin->timer_calls < MAX_TIMER_CALLS)
+                        spin->timer_calls++;
+                      else 
+                        {
+                          spin->timer_calls = 0;
+                          spin->timer_step += spin->climb_rate;
+                        }
+                    }
+                }
+              retval = TRUE;
+            }
+          break;
 
-    case GDK_Page_Up:
+        case GDK_KP_Page_Up:
+        case GDK_Page_Up:
 
-      if (event->state & GDK_CONTROL_MASK)
-	{
-	  gdouble diff = spin->adjustment->upper - spin->adjustment->value;
-	  if (diff > EPSILON)
-	    gtk_spin_button_real_spin (spin, diff);
-	}
-      else
-	gtk_spin_button_real_spin (spin, spin->adjustment->page_increment);
-      return TRUE;
+          if (event->state & GDK_CONTROL_MASK)
+            {
+              gdouble diff = spin->adjustment->upper - spin->adjustment->value;
+              if (diff > EPSILON)
+                gtk_spin_button_real_spin (spin, diff);
+            }
+          else
+            gtk_spin_button_real_spin (spin, spin->adjustment->page_increment);
 
-    case GDK_Page_Down:
+          retval = TRUE;
+          break;
+          
+        case GDK_KP_Page_Down:
+        case GDK_Page_Down:
 
-      if (event->state & GDK_CONTROL_MASK)
-	{
-	  gdouble diff = spin->adjustment->value - spin->adjustment->lower;
-	  if (diff > EPSILON)
-	    gtk_spin_button_real_spin (spin, -diff);
-	}
-      else
-	gtk_spin_button_real_spin (spin, -spin->adjustment->page_increment);
-      return TRUE;
+          if (event->state & GDK_CONTROL_MASK)
+            {
+              gdouble diff = spin->adjustment->value - spin->adjustment->lower;
+              if (diff > EPSILON)
+                gtk_spin_button_real_spin (spin, -diff);
+            }
+          else
+            gtk_spin_button_real_spin (spin, -spin->adjustment->page_increment);
 
-    default:
-      break;
+          retval = TRUE;
+          break;
+
+        default:
+          break;
+        }
     }
 
-  return GTK_WIDGET_CLASS (parent_class)->key_press_event (widget, event);
+  if (retval)
+    {
+      gtk_spin_button_update (spin);
+      return TRUE;
+    }
+  else
+    return GTK_WIDGET_CLASS (parent_class)->key_press_event (widget, event);
 }
 
 static gint
@@ -1269,7 +1262,6 @@ gtk_spin_button_key_release (GtkWidget   *widget,
 {
   GtkSpinButton *spin;
 
-  g_return_val_if_fail (widget != NULL, FALSE);
   g_return_val_if_fail (GTK_IS_SPIN_BUTTON (widget), FALSE);
   
   spin = GTK_SPIN_BUTTON (widget);
@@ -1399,7 +1391,6 @@ gtk_spin_button_real_spin (GtkSpinButton *spin_button,
   GtkAdjustment *adj;
   gdouble new_value = 0.0;
 
-  g_return_if_fail (spin_button != NULL);
   g_return_if_fail (GTK_IS_SPIN_BUTTON (spin_button));
   
   adj = spin_button->adjustment;
@@ -1451,11 +1442,11 @@ gtk_spin_button_default_input (GtkSpinButton *spin_button,
 static gint
 gtk_spin_button_default_output (GtkSpinButton *spin_button)
 {
-  gchar buf[MAX_TEXT_LENGTH];
+  gchar *buf = g_strdup_printf ("%0.*f", spin_button->digits, spin_button->adjustment->value);
 
-  sprintf (buf, "%0.*f", spin_button->digits, spin_button->adjustment->value);
   if (strcmp (buf, gtk_entry_get_text (GTK_ENTRY (spin_button))))
     gtk_entry_set_text (GTK_ENTRY (spin_button), buf);
+  g_free (buf);
   return FALSE;
 }
 
@@ -1473,7 +1464,6 @@ gtk_spin_button_configure (GtkSpinButton  *spin_button,
 			   gdouble         climb_rate,
 			   guint           digits)
 {
-  g_return_if_fail (spin_button != NULL);
   g_return_if_fail (GTK_IS_SPIN_BUTTON (spin_button));
 
   if (adjustment)
@@ -1481,6 +1471,7 @@ gtk_spin_button_configure (GtkSpinButton  *spin_button,
   else
     adjustment = spin_button->adjustment;
 
+  g_object_freeze_notify (G_OBJECT (spin_button));
   if (spin_button->digits != digits) 
     {
       spin_button->digits = digits;
@@ -1492,6 +1483,7 @@ gtk_spin_button_configure (GtkSpinButton  *spin_button,
       spin_button->climb_rate = climb_rate;
       g_object_notify (G_OBJECT (spin_button), "climb_rate");
     }
+  g_object_thaw_notify (G_OBJECT (spin_button));
 
   gtk_adjustment_value_changed (adjustment);
 }
@@ -1547,8 +1539,8 @@ gtk_spin_button_new_with_range (gdouble min,
     digits = 0;
   else {
     digits = abs ((gint) floor (log10 (fabs (step))));
-    if (digits > 5)
-      digits = 5;
+    if (digits > MAX_DIGITS)
+      digits = MAX_DIGITS;
   }
 
   gtk_spin_button_configure (spin, GTK_ADJUSTMENT (adj), step, digits);
@@ -1632,7 +1624,7 @@ gtk_spin_button_get_adjustment (GtkSpinButton *spin_button)
  * @spin_button: a #GtkSpinButton
  * @digits: the number of digits to be displayed for the spin button's value
  * 
- * Set the precision to be displayed by @spin_button. Up to 5 digit precision
+ * Set the precision to be displayed by @spin_button. Up to 20 digit precision
  * is allowed.
  **/
 void
@@ -1652,6 +1644,22 @@ gtk_spin_button_set_digits (GtkSpinButton *spin_button,
     }
 }
 
+/**
+ * gtk_spin_button_get_digits:
+ * @spin_button: a #GtkSpinButton
+ *
+ * Fetches the precision of @spin_button. See gtk_spin_button_set_digits().
+ *
+ * Returns: the current precision
+ **/
+guint
+gtk_spin_button_get_digits (GtkSpinButton *spin_button)
+{
+  g_return_val_if_fail (GTK_IS_SPIN_BUTTON (spin_button), 0);
+
+  return spin_button->digits;
+}
+
 /**
  * gtk_spin_button_set_increments:
  * @spin_button: a #GtkSpinButton
@@ -1672,6 +1680,28 @@ gtk_spin_button_set_increments (GtkSpinButton *spin_button,
   spin_button->adjustment->page_increment = page;
 }
 
+/**
+ * gtk_spin_button_get_increments:
+ * @psin_button: a #GtkSpinButton
+ * @step: location to store step increment, or %NULL
+ * @page: location to store page increment, or %NULL
+ *
+ * Gets the current step and page the increments used by @spin_button. See
+ * gtk_spin_button_set_increments().
+ **/
+void
+gtk_spin_button_get_increments (GtkSpinButton *spin_button,
+				gdouble       *step,
+				gdouble       *page)
+{
+  g_return_if_fail (GTK_IS_SPIN_BUTTON (spin_button));
+
+  if (step)
+    *step = spin_button->adjustment->step_increment;
+  if (page)
+    *page = spin_button->adjustment->page_increment;
+}
+
 /**
  * gtk_spin_button_set_range:
  * @spin_button: a #GtkSpinButton
@@ -1702,6 +1732,28 @@ gtk_spin_button_set_range (GtkSpinButton *spin_button,
   gtk_adjustment_changed (spin_button->adjustment);
 }
 
+/**
+ * gtk_spin_button_get_range:
+ * @spin_button: a #GtkSpinButton
+ * @min: location to store minimum allowed value, or %NULL
+ * @max: location to store maximum allowed value, or %NULL
+ *
+ * Gets the range allowed for @spin_button. See
+ * gtk_spin_button_set_range().
+ **/
+void
+gtk_spin_button_get_range (GtkSpinButton *spin_button,
+			   gdouble       *min,
+			   gdouble       *max)
+{
+  g_return_if_fail (GTK_IS_SPIN_BUTTON (spin_button));
+
+  if (min)
+    *min = spin_button->adjustment->lower;
+  if (max)
+    *max = spin_button->adjustment->upper;
+}
+
 /**
  * gtk_spin_button_get_value:
  * @spin_button: a #GtkSpinButton
@@ -1786,6 +1838,23 @@ gtk_spin_button_set_update_policy (GtkSpinButton             *spin_button,
     }
 }
 
+/**
+ * gtk_spin_button_get_update_policy:
+ * @spin_button: a #GtkSpinButton
+ *
+ * Gets the update behavior of a spin button. See
+ * gtk_spin_button_set_update_policy().
+ *
+ * Return value: the current update policy
+ **/
+GtkSpinButtonUpdatePolicy
+gtk_spin_button_get_update_policy (GtkSpinButton *spin_button)
+{
+  g_return_val_if_fail (GTK_IS_SPIN_BUTTON (spin_button), GTK_UPDATE_ALWAYS);
+
+  return spin_button->update_policy;
+}
+
 /**
  * gtk_spin_button_set_numeric:
  * @spin_button: a #GtkSpinButton 
@@ -1805,6 +1874,23 @@ gtk_spin_button_set_numeric (GtkSpinButton  *spin_button,
   g_object_notify (G_OBJECT (spin_button), "numeric");
 }
 
+/**
+ * gtk_spin_button_get_numeric:
+ * @spin_button: a #GtkSpinButton
+ *
+ * Returns whether non-numeric text can be typed into the spin button.
+ * See gtk_spin_button_set_numeric().
+ *
+ * Return value: %TRUE if only numeric text can be entered
+ **/
+gboolean
+gtk_spin_button_get_numeric (GtkSpinButton *spin_button)
+{
+  g_return_val_if_fail (GTK_IS_SPIN_BUTTON (spin_button), FALSE);
+
+  return spin_button->numeric;
+}
+
 /**
  * gtk_spin_button_set_wrap:
  * @spin_button: a #GtkSpinButton 
@@ -1824,6 +1910,24 @@ gtk_spin_button_set_wrap (GtkSpinButton  *spin_button,
   g_object_notify (G_OBJECT (spin_button), "wrap");
 }
 
+/**
+ * gtk_spin_button_get_wrap:
+ * @spin_button: a #GtkSpinButton
+ *
+ * Returns whether the spin button's value wraps around to the
+ * opposite limit when the upper or lower limit of the range is
+ * exceeded. See gtk_spin_button_set_wrap().
+ *
+ * Return value: %TRUE if the spin button wraps around
+ **/
+gboolean
+gtk_spin_button_get_wrap (GtkSpinButton *spin_button)
+{
+  g_return_val_if_fail (GTK_IS_SPIN_BUTTON (spin_button), FALSE);
+
+  return spin_button->wrap;
+}
+
 /**
  * spin_button_get_shadow_type:
  * @spin_button: a #GtkSpinButton 
@@ -1871,6 +1975,23 @@ gtk_spin_button_set_snap_to_ticks (GtkSpinButton *spin_button,
     }
 }
 
+/**
+ * gtk_spin_button_get_snap_to_ticks:
+ * @spin_button: a #GtkSpinButton
+ *
+ * Returns whether the values are corrected to the nearest step. See
+ * gtk_spin_button_set_snap_to_ticks().
+ *
+ * Return value: %TRUE if values are snapped to the nearest step.
+ **/
+gboolean
+gtk_spin_button_get_snap_to_ticks (GtkSpinButton *spin_button)
+{
+  g_return_val_if_fail (GTK_IS_SPIN_BUTTON (spin_button), FALSE);
+
+  return spin_button->snap_to_ticks;
+}
+
 /**
  * gtk_spin_button_spin:
  * @spin_button: a #GtkSpinButton 
