@@ -1,5 +1,5 @@
 /* gtktreestore.c
- * Copyright (C) 2000  Red Hat, Inc.,  Jonathan Blandford <jrb@redhat.com>
+  * Copyright (C) 2000  Red Hat, Inc.,  Jonathan Blandford <jrb@redhat.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -26,6 +26,7 @@
 
 #define G_NODE(node) ((GNode *)node)
 #define GTK_TREE_STORE_IS_SORTED(tree) (GTK_TREE_STORE (tree)->sort_column_id != -1)
+#define VALID_ITER(iter, tree_store) (iter!= NULL && iter->user_data != NULL && tree_store->stamp == iter->stamp)
 
 static void         gtk_tree_store_init            (GtkTreeStore      *tree_store);
 static void         gtk_tree_store_class_init      (GtkTreeStoreClass *tree_store_class);
@@ -37,6 +38,9 @@ static guint        gtk_tree_store_get_flags       (GtkTreeModel      *tree_mode
 static gint         gtk_tree_store_get_n_columns   (GtkTreeModel      *tree_model);
 static GType        gtk_tree_store_get_column_type (GtkTreeModel      *tree_model,
 						    gint               index);
+static gboolean     gtk_tree_store_get_iter        (GtkTreeModel      *tree_model,
+						    GtkTreeIter       *iter,
+						    GtkTreePath       *path);
 static GtkTreePath *gtk_tree_store_get_path        (GtkTreeModel      *tree_model,
 						    GtkTreeIter       *iter);
 static void         gtk_tree_store_get_value       (GtkTreeModel      *tree_model,
@@ -61,6 +65,13 @@ static gboolean     gtk_tree_store_iter_parent     (GtkTreeModel      *tree_mode
 						    GtkTreeIter       *child);
 
 
+static void gtk_tree_store_set_n_columns   (GtkTreeStore *tree_store,
+					    gint          n_columns);
+static void gtk_tree_store_set_column_type (GtkTreeStore *tree_store,
+					    gint          column,
+					    GType         type);
+
+
 /* DND interfaces */
 static gboolean gtk_tree_store_drag_data_delete   (GtkTreeDragSource *drag_source,
 						   GtkTreePath       *path);
@@ -83,16 +94,20 @@ static void     gtk_tree_store_sort_iter_changed       (GtkTreeStore           *
 							gint                    column);
 static gboolean gtk_tree_store_get_sort_column_id      (GtkTreeSortable        *sortable,
 							gint                   *sort_column_id,
-							GtkTreeSortOrder       *order);
+							GtkSortType            *order);
 static void     gtk_tree_store_set_sort_column_id      (GtkTreeSortable        *sortable,
 							gint                    sort_column_id,
-							GtkTreeSortOrder        order);
-static void     gtk_tree_store_sort_column_id_set_func (GtkTreeSortable        *sortable,
+							GtkSortType             order);
+static void     gtk_tree_store_set_sort_func           (GtkTreeSortable        *sortable,
 							gint                    sort_column_id,
 							GtkTreeIterCompareFunc  func,
 							gpointer                data,
 							GtkDestroyNotify        destroy);
-
+static void     gtk_tree_store_set_default_sort_func   (GtkTreeSortable        *sortable,
+							GtkTreeIterCompareFunc  func,
+							gpointer                data,
+							GtkDestroyNotify        destroy);
+static gboolean gtk_tree_store_has_default_sort_func   (GtkTreeSortable        *sortable);
 
 static void     validate_gnode                    (GNode *node);
 
@@ -190,6 +205,7 @@ gtk_tree_store_tree_model_init (GtkTreeModelIface *iface)
   iface->get_flags = gtk_tree_store_get_flags;
   iface->get_n_columns = gtk_tree_store_get_n_columns;
   iface->get_column_type = gtk_tree_store_get_column_type;
+  iface->get_iter = gtk_tree_store_get_iter;
   iface->get_path = gtk_tree_store_get_path;
   iface->get_value = gtk_tree_store_get_value;
   iface->iter_next = gtk_tree_store_iter_next;
@@ -219,7 +235,9 @@ gtk_tree_store_sortable_init (GtkTreeSortableIface *iface)
 {
   iface->get_sort_column_id = gtk_tree_store_get_sort_column_id;
   iface->set_sort_column_id = gtk_tree_store_set_sort_column_id;
-  iface->sort_column_id_set_func = gtk_tree_store_sort_column_id_set_func;
+  iface->set_sort_func = gtk_tree_store_set_sort_func;
+  iface->set_default_sort_func = gtk_tree_store_set_default_sort_func;
+  iface->has_default_sort_func = gtk_tree_store_has_default_sort_func;
 }
 
 static void
@@ -231,14 +249,20 @@ gtk_tree_store_init (GtkTreeStore *tree_store)
   tree_store->sort_column_id = -1;
 }
 
+/**
+ * gtk_tree_store_new:
+ * @n_columns: number of columns in the tree store
+ * @Varargs: all #GType types for the columns, from first to last
+ *
+ * Creates a new tree store as with @n_columns columns each of the types passed
+ * in.  As an example, gtk_tree_store_new (3, G_TYPE_INT, G_TYPE_STRING,
+ * GDK_TYPE_PIXBUF); will create a new GtkTreeStore with three columns, of type
+ * int, string and GDkPixbuf respectively.
+ *
+ * Return value: a new #GtkTreeStore
+ **/
 GtkTreeStore *
-gtk_tree_store_new (void)
-{
-  return GTK_TREE_STORE (g_object_new (GTK_TYPE_TREE_STORE, NULL));
-}
-
-GtkTreeStore *
-gtk_tree_store_new_with_types (gint n_columns,
+gtk_tree_store_new (gint n_columns,
 			       ...)
 {
   GtkTreeStore *retval;
@@ -247,7 +271,7 @@ gtk_tree_store_new_with_types (gint n_columns,
 
   g_return_val_if_fail (n_columns > 0, NULL);
 
-  retval = gtk_tree_store_new ();
+  retval = GTK_TREE_STORE (g_object_new (GTK_TYPE_TREE_STORE, NULL));
   gtk_tree_store_set_n_columns (retval, n_columns);
 
   va_start (args, n_columns);
@@ -267,22 +291,55 @@ gtk_tree_store_new_with_types (gint n_columns,
 
   return retval;
 }
+/**
+ * gtk_tree_store_newv:
+ * @n_columns: number of columns in the tree store
+ * @types: an array of #GType types for the columns, from first to last
+ *
+ * Non vararg creation function.  Used primarily by language bindings.
+ *
+ * Return value: a new #GtkTreeStore
+ **/
+GtkTreeStore *
+gtk_tree_store_newv (gint   n_columns,
+		     GType *types)
+{
+  GtkTreeStore *retval;
+  gint i;
+
+  g_return_val_if_fail (n_columns > 0, NULL);
+
+  retval = GTK_TREE_STORE (g_object_new (GTK_TYPE_TREE_STORE, NULL));
+  gtk_tree_store_set_n_columns (retval, n_columns);
+
+   for (i = 0; i < n_columns; i++)
+    {
+      if (! _gtk_tree_data_list_check_type (types[i]))
+	{
+	  g_warning ("%s: Invalid type %s passed to gtk_tree_store_new_with_types\n", G_STRLOC, g_type_name (types[i]));
+	  g_object_unref (G_OBJECT (retval));
+	  return NULL;
+	}
+      gtk_tree_store_set_column_type (retval, i, types[i]);
+    }
+
+  return retval;
+}
 
 /**
  * gtk_tree_store_set_n_columns:
- * @tree_store: 
- * @n_columns: 
- * 
+ * @tree_store:
+ * @n_columns:
+ *
  * As a side effect of calling this function, all sort columns that overlap with
  * the current number of columns will be removed.
  **/
-void
+static void
 gtk_tree_store_set_n_columns (GtkTreeStore *tree_store,
 			      gint          n_columns)
 {
   GType *new_columns;
 
-  g_return_if_fail (tree_store != NULL);
   g_return_if_fail (GTK_IS_TREE_STORE (tree_store));
 
   if (tree_store->n_columns == n_columns)
@@ -321,12 +378,11 @@ gtk_tree_store_set_n_columns (GtkTreeStore *tree_store,
  * subclasses of those types such as %GDK_TYPE_PIXBUF.
  *
  **/
-void
+static void
 gtk_tree_store_set_column_type (GtkTreeStore *tree_store,
 				gint          column,
 				GType         type)
 {
-  g_return_if_fail (tree_store != NULL);
   g_return_if_fail (GTK_IS_TREE_STORE (tree_store));
   g_return_if_fail (column >=0 && column < tree_store->n_columns);
   if (!_gtk_tree_data_list_check_type (type))
@@ -371,6 +427,39 @@ gtk_tree_store_get_column_type (GtkTreeModel *tree_model,
   return GTK_TREE_STORE (tree_model)->column_headers[index];
 }
 
+static gboolean
+gtk_tree_store_get_iter (GtkTreeModel *tree_model,
+			 GtkTreeIter  *iter,
+			 GtkTreePath  *path)
+{
+  GtkTreeStore *tree_store = (GtkTreeStore *) tree_model;
+  GtkTreeIter parent;
+  gint *indices;
+  gint depth, i;
+
+  g_return_val_if_fail (GTK_IS_TREE_STORE (tree_store), FALSE);
+  
+  indices = gtk_tree_path_get_indices (path);
+  depth = gtk_tree_path_get_depth (path);
+
+  g_return_val_if_fail (depth > 0, FALSE);
+
+  parent.stamp = tree_store->stamp;
+  parent.user_data = tree_store->root;
+
+  if (! gtk_tree_model_iter_nth_child (tree_model, iter, &parent, indices[0]))
+    return FALSE;
+
+  for (i = 1; i < depth; i++)
+    {
+      parent = *iter;
+      if (! gtk_tree_model_iter_nth_child (tree_model, iter, &parent, indices[i]))
+	return FALSE;
+    }
+
+  return TRUE;
+}
+
 static GtkTreePath *
 gtk_tree_store_get_path (GtkTreeModel *tree_model,
 			 GtkTreeIter  *iter)
@@ -379,13 +468,16 @@ gtk_tree_store_get_path (GtkTreeModel *tree_model,
   GNode *tmp_node;
   gint i = 0;
 
-  g_return_val_if_fail (tree_model != NULL, NULL);
   g_return_val_if_fail (GTK_IS_TREE_STORE (tree_model), NULL);
   g_return_val_if_fail (iter != NULL, NULL);
   g_return_val_if_fail (iter->user_data != NULL, NULL);
+  g_return_val_if_fail (iter->stamp == GTK_TREE_STORE (tree_model)->stamp, NULL);
 
   validate_tree ((GtkTreeStore*)tree_model);
 
+  if (G_NODE (iter->user_data)->parent == NULL &&
+      G_NODE (iter->user_data) == GTK_TREE_STORE (tree_model)->root)
+    return gtk_tree_path_new ();
   g_assert (G_NODE (iter->user_data)->parent != NULL);
 
   if (G_NODE (iter->user_data)->parent == G_NODE (GTK_TREE_STORE (tree_model)->root))
@@ -443,7 +535,6 @@ gtk_tree_store_get_value (GtkTreeModel *tree_model,
   GtkTreeDataList *list;
   gint tmp_column = column;
 
-  g_return_if_fail (tree_model != NULL);
   g_return_if_fail (GTK_IS_TREE_STORE (tree_model));
   g_return_if_fail (iter != NULL);
   g_return_if_fail (column < GTK_TREE_STORE (tree_model)->n_columns);
@@ -509,7 +600,6 @@ static gboolean
 gtk_tree_store_iter_has_child (GtkTreeModel *tree_model,
 			       GtkTreeIter  *iter)
 {
-  g_return_val_if_fail (tree_model != NULL, FALSE);
   g_return_val_if_fail (GTK_IS_TREE_STORE (tree_model), FALSE);
   g_return_val_if_fail (iter != NULL, FALSE);
   g_return_val_if_fail (iter->user_data != NULL, FALSE);
@@ -524,10 +614,8 @@ gtk_tree_store_iter_n_children (GtkTreeModel *tree_model,
   GNode *node;
   gint i = 0;
 
-  g_return_val_if_fail (tree_model != NULL, 0);
   g_return_val_if_fail (GTK_IS_TREE_STORE (tree_model), 0);
-  g_return_val_if_fail (iter != NULL, FALSE);
-  g_return_val_if_fail (iter->user_data != NULL, FALSE);
+  g_return_val_if_fail (iter == NULL || iter->user_data != NULL, FALSE);
 
   if (iter == NULL)
     node = G_NODE (GTK_TREE_STORE (tree_model)->root)->children;
@@ -552,7 +640,6 @@ gtk_tree_store_iter_nth_child (GtkTreeModel *tree_model,
   GNode *parent_node;
   GNode *child;
 
-  g_return_val_if_fail (tree_model != NULL, FALSE);
   g_return_val_if_fail (GTK_IS_TREE_STORE (tree_model), FALSE);
   g_return_val_if_fail (parent == NULL || parent->user_data != NULL, FALSE);
 
@@ -612,9 +699,10 @@ gtk_tree_store_set_value (GtkTreeStore *tree_store,
   GtkTreePath *path = NULL;
   GValue real_value = {0, };
   gboolean converted = FALSE;
+  gint orig_column = column;
 
-  g_return_if_fail (tree_store != NULL);
   g_return_if_fail (GTK_IS_TREE_STORE (tree_store));
+  g_return_if_fail (VALID_ITER (iter, tree_store));
   g_return_if_fail (column >= 0 && column < tree_store->n_columns);
   g_return_if_fail (G_IS_VALUE (value));
 
@@ -693,7 +781,7 @@ gtk_tree_store_set_value (GtkTreeStore *tree_store,
     g_value_unset (&real_value);
 
   if (GTK_TREE_STORE_IS_SORTED (tree_store))
-    gtk_tree_store_sort_iter_changed (tree_store, iter, column);
+    gtk_tree_store_sort_iter_changed (tree_store, iter, orig_column);
 }
 
 /**
@@ -714,6 +802,7 @@ gtk_tree_store_set_valist (GtkTreeStore *tree_store,
   gint column;
 
   g_return_if_fail (GTK_IS_TREE_STORE (tree_store));
+  g_return_if_fail (VALID_ITER (iter, tree_store));
 
   column = va_arg (var_args, gint);
 
@@ -774,6 +863,7 @@ gtk_tree_store_set (GtkTreeStore *tree_store,
   va_list var_args;
 
   g_return_if_fail (GTK_IS_TREE_STORE (tree_store));
+  g_return_if_fail (VALID_ITER (iter, tree_store));
 
   va_start (var_args, iter);
   gtk_tree_store_set_valist (tree_store, iter, var_args);
@@ -788,8 +878,8 @@ gtk_tree_store_remove (GtkTreeStore *model,
   GtkTreeIter new_iter = {0,};
   GNode *parent;
 
-  g_return_if_fail (model != NULL);
   g_return_if_fail (GTK_IS_TREE_STORE (model));
+  g_return_if_fail (VALID_ITER (iter, model));
 
   parent = G_NODE (iter->user_data)->parent;
 
@@ -805,13 +895,29 @@ gtk_tree_store_remove (GtkTreeStore *model,
   model->stamp++;
   gtk_tree_model_deleted (GTK_TREE_MODEL (model), path);
 
-  if (parent != G_NODE (model->root) && parent->children == NULL)
+  if (parent != G_NODE (model->root))
     {
-      gtk_tree_path_up (path);
+      /* child_toggled */
+      if (parent->children == NULL)
+	{
+	  gtk_tree_path_up (path);
 
-      new_iter.stamp = model->stamp;
-      new_iter.user_data = parent;
-      gtk_tree_model_has_child_toggled (GTK_TREE_MODEL (model), path, &new_iter);
+	  new_iter.stamp = model->stamp;
+	  new_iter.user_data = parent;
+	  gtk_tree_model_has_child_toggled (GTK_TREE_MODEL (model), path, &new_iter);
+	}
+
+      /* revalidate iter */
+      while (parent != G_NODE (model->root))
+	{
+	  if (parent->next != NULL)
+	    {
+	      iter->stamp = model->stamp;
+	      iter->user_data = parent->next;
+	      break;
+	    }
+	  parent = parent->parent;
+	}
     }
   gtk_tree_path_free (path);
 }
@@ -825,8 +931,9 @@ gtk_tree_store_insert (GtkTreeStore *model,
   GtkTreePath *path;
   GNode *parent_node;
 
-  g_return_if_fail (model != NULL);
   g_return_if_fail (GTK_IS_TREE_STORE (model));
+  if (parent)
+    g_return_if_fail (VALID_ITER (parent, model));
 
   if (parent)
     parent_node = parent->user_data;
@@ -855,9 +962,12 @@ gtk_tree_store_insert_before (GtkTreeStore *model,
   GNode *parent_node = NULL;
   GNode *new_node;
 
-  g_return_if_fail (model != NULL);
   g_return_if_fail (GTK_IS_TREE_STORE (model));
   g_return_if_fail (iter != NULL);
+  if (parent != NULL)
+    g_return_if_fail (VALID_ITER (parent, model));
+  if (sibling != NULL)
+    g_return_if_fail (VALID_ITER (sibling, model));
 
   new_node = g_node_new (NULL);
 
@@ -869,8 +979,7 @@ gtk_tree_store_insert_before (GtkTreeStore *model,
     parent_node = G_NODE (parent->user_data);
   else
     {
-      g_return_if_fail (G_NODE (sibling->user_data)->parent ==
-                        G_NODE (parent->user_data));
+      g_return_if_fail (G_NODE (sibling->user_data)->parent == G_NODE (parent->user_data));
       parent_node = G_NODE (parent->user_data);
     }
 
@@ -899,9 +1008,12 @@ gtk_tree_store_insert_after (GtkTreeStore *model,
   GNode *parent_node;
   GNode *new_node;
 
-  g_return_if_fail (model != NULL);
   g_return_if_fail (GTK_IS_TREE_STORE (model));
   g_return_if_fail (iter != NULL);
+  if (parent != NULL)
+    g_return_if_fail (VALID_ITER (parent, model));
+  if (sibling != NULL)
+    g_return_if_fail (VALID_ITER (sibling, model));
 
   new_node = g_node_new (NULL);
 
@@ -941,9 +1053,10 @@ gtk_tree_store_prepend (GtkTreeStore *model,
 {
   GNode *parent_node;
 
-  g_return_if_fail (model != NULL);
   g_return_if_fail (GTK_IS_TREE_STORE (model));
   g_return_if_fail (iter != NULL);
+  if (parent != NULL)
+    g_return_if_fail (VALID_ITER (parent, model));
 
   if (parent == NULL)
     parent_node = model->root;
@@ -987,10 +1100,12 @@ gtk_tree_store_append (GtkTreeStore *model,
 {
   GNode *parent_node;
 
-  g_return_if_fail (model != NULL);
   g_return_if_fail (GTK_IS_TREE_STORE (model));
   g_return_if_fail (iter != NULL);
 
+  if (parent != NULL)
+    g_return_if_fail (VALID_ITER (parent, model));
+
   if (parent == NULL)
     parent_node = model->root;
   else
@@ -1032,10 +1147,9 @@ gtk_tree_store_is_ancestor (GtkTreeStore *model,
 			    GtkTreeIter  *iter,
 			    GtkTreeIter  *descendant)
 {
-  g_return_val_if_fail (model != NULL, FALSE);
   g_return_val_if_fail (GTK_IS_TREE_STORE (model), FALSE);
-  g_return_val_if_fail (iter != NULL, FALSE);
-  g_return_val_if_fail (descendant != NULL, FALSE);
+  g_return_val_if_fail (VALID_ITER (iter, model), FALSE);
+  g_return_val_if_fail (VALID_ITER (descendant, model), FALSE);
 
   return g_node_is_ancestor (G_NODE (iter->user_data),
 			     G_NODE (descendant->user_data));
@@ -1046,15 +1160,29 @@ gint
 gtk_tree_store_iter_depth (GtkTreeStore *model,
 			   GtkTreeIter  *iter)
 {
-  g_return_val_if_fail (model != NULL, 0);
   g_return_val_if_fail (GTK_IS_TREE_STORE (model), 0);
-  g_return_val_if_fail (iter != NULL, 0);
+  g_return_val_if_fail (VALID_ITER (iter, model), 0);
 
   return g_node_depth (G_NODE (iter->user_data)) - 1;
 }
 
-/* DND */
 
+void
+gtk_tree_store_clear (GtkTreeStore *tree_store)
+{
+  GtkTreeIter iter;
+
+  g_return_if_fail (GTK_IS_TREE_STORE (tree_store));
+
+  while (G_NODE (tree_store->root)->children)
+    {
+      iter.stamp = tree_store->stamp;
+      iter.user_data = G_NODE (tree_store->root)->children;
+      gtk_tree_store_remove (tree_store, &iter);
+    }
+}
+
+/* DND */
 
 
 static gboolean
@@ -1330,11 +1458,160 @@ gtk_tree_store_row_drop_possible (GtkTreeDragDest *drag_dest,
   return TRUE;
 }
 
+/* Sorting */
+typedef struct _SortTuple
+{
+  gint offset;
+  GNode *node;
+} SortTuple;
+
+static gint
+gtk_tree_store_compare_func (gconstpointer a,
+			     gconstpointer b,
+			     gpointer      user_data)
+{
+  GtkTreeStore *tree_store = user_data;
+  GNode *node_a;
+  GNode *node_b;
+  GtkTreeIterCompareFunc func;
+  gpointer data;
+
+  GtkTreeIter iter_a;
+  GtkTreeIter iter_b;
+  gint retval;
+
+  if (tree_store->sort_column_id != -1)
+    {
+      GtkTreeDataSortHeader *header;
+
+      header = _gtk_tree_data_list_get_header (tree_store->sort_list,
+					       tree_store->sort_column_id);
+      g_return_val_if_fail (header != NULL, 0);
+      g_return_val_if_fail (header->func != NULL, 0);
+
+      func = header->func;
+      data = header->data;
+    }
+  else
+    {
+      g_return_val_if_fail (tree_store->default_sort_func != NULL, 0);
+      func = tree_store->default_sort_func;
+      data = tree_store->default_sort_data;
+    }
+
+  node_a = ((SortTuple *) a)->node;
+  node_b = ((SortTuple *) b)->node;
+
+  iter_a.stamp = tree_store->stamp;
+  iter_a.user_data = node_a;
+  iter_b.stamp = tree_store->stamp;
+  iter_b.user_data = node_b;
+
+  retval = (* func) (GTK_TREE_MODEL (user_data), &iter_a, &iter_b, data);
+
+  if (tree_store->order == GTK_SORT_DESCENDING)
+    {
+      if (retval > 0)
+	retval = -1;
+      else if (retval < 0)
+	retval = 1;
+    }
+  return retval;
+}
+
+static void
+gtk_tree_store_sort_helper (GtkTreeStore *tree_store,
+			    GNode        *parent,
+			    gboolean      recurse)
+{
+  GtkTreeIter iter;
+  GArray *sort_array;
+  GNode *node;
+  GNode *tmp_node;
+  gint list_length;
+  gint i;
+  gint *new_order;
+  GtkTreePath *path;
+
+  node = parent->children;
+  if (node->next == NULL)
+    return;
+
+  g_assert (GTK_TREE_STORE_IS_SORTED (tree_store));
+
+  list_length = 0;
+  for (tmp_node = node; tmp_node; tmp_node = tmp_node->next)
+    list_length++;
+
+  sort_array = g_array_sized_new (FALSE, FALSE, sizeof (SortTuple), list_length);
+
+  i = 0;
+  for (tmp_node = node; tmp_node; tmp_node = tmp_node->next)
+    {
+      SortTuple tuple;
+
+      tuple.offset = i;
+      tuple.node = tmp_node;
+      g_array_append_val (sort_array, tuple);
+      i++;
+    }
+
+  g_array_sort_with_data (sort_array, gtk_tree_store_compare_func, tree_store);
+
+  for (i = 0; i < list_length - 1; i++)
+    {
+      g_array_index (sort_array, SortTuple, i).node->next =
+	g_array_index (sort_array, SortTuple, i + 1).node;
+      g_array_index (sort_array, SortTuple, i + 1).node->prev =
+	g_array_index (sort_array, SortTuple, i).node;
+    }
+  g_array_index (sort_array, SortTuple, list_length - 1).node->next = NULL;
+  g_array_index (sort_array, SortTuple, 0).node->prev = NULL;
+  parent->children = g_array_index (sort_array, SortTuple, 0).node;
+
+  /* Let the world know about our new order */
+  new_order = g_new (gint, list_length);
+  for (i = 0; i < list_length; i++)
+    new_order[i] = g_array_index (sort_array, SortTuple, i).offset;
+
+  iter.stamp = tree_store->stamp;
+  iter.user_data = parent;
+  path = gtk_tree_store_get_path (GTK_TREE_MODEL (tree_store), &iter);
+  gtk_tree_model_reordered (GTK_TREE_MODEL (tree_store),
+			    path, &iter, new_order);
+  gtk_tree_path_free (path);
+  g_free (new_order);
+  g_array_free (sort_array, TRUE);
+
+  if (recurse)
+    {
+      for (tmp_node = parent->children; tmp_node; tmp_node = tmp_node->next)
+	{
+	  if (tmp_node->children)
+	    gtk_tree_store_sort_helper (tree_store, tmp_node, TRUE);
+	}
+    }
+}
 
 static void
 gtk_tree_store_sort (GtkTreeStore *tree_store)
 {
+  if (tree_store->sort_column_id != -1)
+    {
+      GtkTreeDataSortHeader *header = NULL;
 
+      header = _gtk_tree_data_list_get_header (tree_store->sort_list, tree_store->sort_column_id);
+
+      /* We want to make sure that we have a function */
+      g_return_if_fail (header != NULL);
+      g_return_if_fail (header->func != NULL);
+    }
+  else
+    {
+      g_return_if_fail (tree_store->default_sort_func != NULL);
+    }
+
+  gtk_tree_store_sort_helper (tree_store, G_NODE (tree_store->root), TRUE);
 }
 
 static void
@@ -1342,7 +1619,6 @@ gtk_tree_store_sort_iter_changed (GtkTreeStore *tree_store,
 				  GtkTreeIter  *iter,
 				  gint          column)
 {
-  GtkTreeDataSortHeader *header;
   GNode *prev = NULL;
   GNode *next = NULL;
   GNode *node;
@@ -1355,57 +1631,64 @@ gtk_tree_store_sort_iter_changed (GtkTreeStore *tree_store,
   gint new_location;
   gint *new_order;
   gint length;
+  GtkTreeIterCompareFunc func;
+  gpointer data;
 
-  /* FIX*/
-  if (G_NODE (iter->user_data)->parent)
-    node = G_NODE (iter->user_data)->parent->children;
-  else
-    node = G_NODE (tree_store->root);
+  g_return_if_fail (G_NODE (iter->user_data)->parent != NULL);
 
   tmp_iter.stamp = tree_store->stamp;
-  header = _gtk_tree_data_list_get_header (tree_store->sort_list,
-					   tree_store->sort_column_id);
-  g_return_if_fail (header != NULL);
-  g_return_if_fail (header->func != NULL);
+  if (tree_store->sort_column_id != -1)
+    {
+      GtkTreeDataSortHeader *header;
+      header = _gtk_tree_data_list_get_header (tree_store->sort_list,
+					       tree_store->sort_column_id);
+      g_return_if_fail (header != NULL);
+      g_return_if_fail (header->func != NULL);
+      func = header->func;
+      data = header->data;
+    }
+  else
+    {
+      g_return_if_fail (tree_store->default_sort_func != NULL);
+      func = tree_store->default_sort_func;
+      data = tree_store->default_sort_data;
+    }
 
   /* If it's the built in function, we don't sort. */
-  if (header->func == gtk_tree_data_list_compare_func &&
+  if (func == gtk_tree_data_list_compare_func &&
       tree_store->sort_column_id != column)
     return;
 
   old_location = 0;
+  node = G_NODE (iter->user_data)->parent->children;
   /* First we find the iter, its prev, and its next */
   while (node)
     {
       if (node == G_NODE (iter->user_data))
 	break;
-      prev = node;
-      node = node->next;
       old_location++;
+      node = node->next;
     }
   g_assert (node != NULL);
 
+  prev = node->prev;
   next = node->next;
 
   /* Check the common case, where we don't need to sort it moved. */
   if (prev != NULL)
     {
       tmp_iter.user_data = prev;
-      cmp_a = (* header->func) (GTK_TREE_MODEL (tree_store),
-				&tmp_iter, iter,
-				header->data);
+      cmp_a = (* func) (GTK_TREE_MODEL (tree_store), &tmp_iter, iter, data);
     }
 
   if (next != NULL)
     {
       tmp_iter.user_data = next;
-      cmp_b = (* header->func) (GTK_TREE_MODEL (tree_store),
-				iter, &tmp_iter,
-				header->data);
+      cmp_b = (* func) (GTK_TREE_MODEL (tree_store), iter, &tmp_iter, data);
     }
 
 
-  if (tree_store->order == GTK_TREE_SORT_DESCENDING)
+  if (tree_store->order == GTK_SORT_DESCENDING)
     {
       if (cmp_a < 0)
 	cmp_a = 1;
@@ -1429,23 +1712,25 @@ gtk_tree_store_sort_iter_changed (GtkTreeStore *tree_store,
   /* We actually need to sort it */
   /* First, remove the old link. */
 
-  if (prev == NULL)
-    tree_store->root = next;
-  else
+  if (prev)
     prev->next = next;
+  else
+    node->parent->children = next;
+  if (next)
+    next->prev = prev;
+
+  node->prev = NULL;
   node->next = NULL;
-  
+
   /* FIXME: as an optimization, we can potentially start at next */
   prev = NULL;
-  node = G_NODE (tree_store->root);
+  node = node->parent->children;
   new_location = 0;
   tmp_iter.user_data = node;
-  if (tree_store->order == GTK_TREE_SORT_DESCENDING)
-    cmp_a = (* header->func) (GTK_TREE_MODEL (tree_store),
-			      &tmp_iter, iter, header->data);
+  if (tree_store->order == GTK_SORT_DESCENDING)
+    cmp_a = (* func) (GTK_TREE_MODEL (tree_store), &tmp_iter, iter, data);
   else
-    cmp_a = (* header->func) (GTK_TREE_MODEL (tree_store),
-			      iter, &tmp_iter, header->data);
+    cmp_a = (* func) (GTK_TREE_MODEL (tree_store), iter, &tmp_iter, data);
 
   while ((node->next) && (cmp_a > 0))
     {
@@ -1453,27 +1738,28 @@ gtk_tree_store_sort_iter_changed (GtkTreeStore *tree_store,
       node = node->next;
       new_location++;
       tmp_iter.user_data = node;
-      if (tree_store->order == GTK_TREE_SORT_DESCENDING)
-	cmp_a = (* header->func) (GTK_TREE_MODEL (tree_store),
-				  &tmp_iter, iter, header->data);
+      if (tree_store->order == GTK_SORT_DESCENDING)
+	cmp_a = (* func) (GTK_TREE_MODEL (tree_store), &tmp_iter, iter, data);
       else
-	cmp_a = (* header->func) (GTK_TREE_MODEL (tree_store),
-				  iter, &tmp_iter, header->data);
+	cmp_a = (* func) (GTK_TREE_MODEL (tree_store), iter, &tmp_iter, data);
     }
 
   if ((!node->next) && (cmp_a > 0))
     {
       node->next = G_NODE (iter->user_data);
+      node->next->prev = node;
     }
   else if (prev)
     {
       prev->next = G_NODE (iter->user_data);
+      prev->next->prev = prev;
       G_NODE (iter->user_data)->next = node;
+      G_NODE (iter->user_data)->next->prev = G_NODE (iter->user_data);
     }
   else
     {
-      G_NODE (iter->user_data)->next = G_NODE (tree_store->root);
-      tree_store->root = G_NODE (iter->user_data);
+      G_NODE (iter->user_data)->next = G_NODE (iter->user_data)->parent->children;
+      G_NODE (iter->user_data)->parent->children = G_NODE (iter->user_data);
     }
 
   /* Emit the reordered signal. */
@@ -1504,9 +1790,8 @@ gtk_tree_store_sort_iter_changed (GtkTreeStore *tree_store,
 	  new_order[i] = old_location;
       }
 
-
-  tmp_path = gtk_tree_path_new ();
-  tmp_iter.user_data = NULL;
+  tmp_iter.user_data = node->parent;
+  tmp_path = gtk_tree_store_get_path (GTK_TREE_MODEL (tree_store), &tmp_iter);
 
   gtk_tree_model_reordered (GTK_TREE_MODEL (tree_store),
 			    tmp_path, &tmp_iter,
@@ -1520,11 +1805,10 @@ gtk_tree_store_sort_iter_changed (GtkTreeStore *tree_store,
 static gboolean
 gtk_tree_store_get_sort_column_id (GtkTreeSortable  *sortable,
 				   gint             *sort_column_id,
-				   GtkTreeSortOrder *order)
+				   GtkSortType      *order)
 {
   GtkTreeStore *tree_store = (GtkTreeStore *) sortable;
 
-  g_return_val_if_fail (sortable != NULL, FALSE);
   g_return_val_if_fail (GTK_IS_TREE_STORE (sortable), FALSE);
 
   if (tree_store->sort_column_id == -1)
@@ -1541,47 +1825,51 @@ gtk_tree_store_get_sort_column_id (GtkTreeSortable  *sortable,
 static void
 gtk_tree_store_set_sort_column_id (GtkTreeSortable  *sortable,
 				   gint              sort_column_id,
-				   GtkTreeSortOrder  order)
+				   GtkSortType       order)
 {
   GtkTreeStore *tree_store = (GtkTreeStore *) sortable;
-  GList *list;
 
-  g_return_if_fail (sortable != NULL);
   g_return_if_fail (GTK_IS_TREE_STORE (sortable));
 
-  for (list = tree_store->sort_list; list; list = list->next)
-    {
-      GtkTreeDataSortHeader *header = (GtkTreeDataSortHeader*) list->data;
-      if (header->sort_column_id == sort_column_id)
-	break;
-    }
-  g_return_if_fail (list != NULL);
-
+  
   if ((tree_store->sort_column_id == sort_column_id) &&
       (tree_store->order == order))
     return;
 
+  if (sort_column_id != -1)
+    {
+      GtkTreeDataSortHeader *header = NULL;
+
+      header = _gtk_tree_data_list_get_header (tree_store->sort_list, sort_column_id);
+
+      /* We want to make sure that we have a function */
+      g_return_if_fail (header != NULL);
+      g_return_if_fail (header->func != NULL);
+    }
+  else
+    {
+      g_return_if_fail (tree_store->default_sort_func != NULL);
+    }
+
   tree_store->sort_column_id = sort_column_id;
   tree_store->order = order;
 
-  if (tree_store->sort_column_id >= 0)
-    gtk_tree_store_sort (tree_store);
+  gtk_tree_store_sort (tree_store);
 
   gtk_tree_sortable_sort_column_changed (sortable);
 }
 
 static void
-gtk_tree_store_sort_column_id_set_func (GtkTreeSortable        *sortable,
-					gint                    sort_column_id,
-					GtkTreeIterCompareFunc  func,
-					gpointer                data,
-					GtkDestroyNotify        destroy)
+gtk_tree_store_set_sort_func (GtkTreeSortable        *sortable,
+			      gint                    sort_column_id,
+			      GtkTreeIterCompareFunc  func,
+			      gpointer                data,
+			      GtkDestroyNotify        destroy)
 {
   GtkTreeStore *tree_store = (GtkTreeStore *) sortable;
   GtkTreeDataSortHeader *header = NULL;
   GList *list;
 
-  g_return_if_fail (sortable != NULL);
   g_return_if_fail (GTK_IS_TREE_STORE (sortable));
   g_return_if_fail (func != NULL);
 
@@ -1608,6 +1896,34 @@ gtk_tree_store_sort_column_id_set_func (GtkTreeSortable        *sortable,
 
 }
 
+static void
+gtk_tree_store_set_default_sort_func (GtkTreeSortable        *sortable,
+				      GtkTreeIterCompareFunc  func,
+				      gpointer                data,
+				      GtkDestroyNotify        destroy)
+{
+  GtkTreeStore *tree_store = (GtkTreeStore *) sortable;
+
+  g_return_if_fail (GTK_IS_TREE_STORE (sortable));
+
+  if (tree_store->default_sort_destroy)
+    (* tree_store->default_sort_destroy) (tree_store->default_sort_data);
+
+  tree_store->default_sort_func = func;
+  tree_store->default_sort_data = data;
+  tree_store->default_sort_destroy = destroy;
+}
+
+static gboolean
+gtk_tree_store_has_default_sort_func (GtkTreeSortable *sortable)
+{
+  GtkTreeStore *tree_store = (GtkTreeStore *) sortable;
+
+  g_return_val_if_fail (GTK_IS_TREE_STORE (sortable), FALSE);
+
+  return (tree_store->default_sort_func != NULL);
+}
+
 static void
 validate_gnode (GNode* node)
 {
@@ -1625,3 +1941,4 @@ validate_gnode (GNode* node)
 }
 
 
+
