@@ -88,7 +88,6 @@ static const GtkEnumValue _gtk_cell_renderer_state_values[] = {
 /* enumerations from "gtkdebug.h" */
 static const GtkEnumValue _gtk_debug_flag_values[] = {
   { GTK_DEBUG_MISC, "GTK_DEBUG_MISC", "misc" },
-  { GTK_DEBUG_DND, "GTK_DEBUG_DND", "dnd" },
   { GTK_DEBUG_PLUGSOCKET, "GTK_DEBUG_PLUGSOCKET", "plugsocket" },
   { GTK_DEBUG_TEXT, "GTK_DEBUG_TEXT", "text" },
   { GTK_DEBUG_TREE, "GTK_DEBUG_TREE", "tree" },
@@ -200,12 +199,20 @@ static const GtkEnumValue _gtk_direction_type_values[] = {
   { GTK_DIR_RIGHT, "GTK_DIR_RIGHT", "right" },
   { 0, NULL, NULL }
 };
+static const GtkEnumValue _gtk_expander_style_values[] = {
+  { GTK_EXPANDER_COLLAPSED, "GTK_EXPANDER_COLLAPSED", "collapsed" },
+  { GTK_EXPANDER_SEMI_COLLAPSED, "GTK_EXPANDER_SEMI_COLLAPSED", "semi-collapsed" },
+  { GTK_EXPANDER_SEMI_EXPANDED, "GTK_EXPANDER_SEMI_EXPANDED", "semi-expanded" },
+  { GTK_EXPANDER_EXPANDED, "GTK_EXPANDER_EXPANDED", "expanded" },
+  { 0, NULL, NULL }
+};
 static const GtkEnumValue _gtk_icon_size_values[] = {
   { GTK_ICON_SIZE_INVALID, "GTK_ICON_SIZE_INVALID", "invalid" },
   { GTK_ICON_SIZE_MENU, "GTK_ICON_SIZE_MENU", "menu" },
   { GTK_ICON_SIZE_SMALL_TOOLBAR, "GTK_ICON_SIZE_SMALL_TOOLBAR", "small-toolbar" },
   { GTK_ICON_SIZE_LARGE_TOOLBAR, "GTK_ICON_SIZE_LARGE_TOOLBAR", "large-toolbar" },
   { GTK_ICON_SIZE_BUTTON, "GTK_ICON_SIZE_BUTTON", "button" },
+  { GTK_ICON_SIZE_DND, "GTK_ICON_SIZE_DND", "dnd" },
   { GTK_ICON_SIZE_DIALOG, "GTK_ICON_SIZE_DIALOG", "dialog" },
   { 0, NULL, NULL }
 };
@@ -284,6 +291,7 @@ static const GtkEnumValue _gtk_path_priority_type_values[] = {
   { GTK_PATH_PRIO_LOWEST, "GTK_PATH_PRIO_LOWEST", "lowest" },
   { GTK_PATH_PRIO_GTK, "GTK_PATH_PRIO_GTK", "gtk" },
   { GTK_PATH_PRIO_APPLICATION, "GTK_PATH_PRIO_APPLICATION", "application" },
+  { GTK_PATH_PRIO_THEME, "GTK_PATH_PRIO_THEME", "theme" },
   { GTK_PATH_PRIO_RC, "GTK_PATH_PRIO_RC", "rc" },
   { GTK_PATH_PRIO_HIGHEST, "GTK_PATH_PRIO_HIGHEST", "highest" },
   { GTK_PATH_PRIO_MASK, "GTK_PATH_PRIO_MASK", "mask" },
@@ -484,14 +492,6 @@ static const GtkEnumValue _gtk_arg_flags_values[] = {
   { 0, NULL, NULL }
 };
 
-/* enumerations from "gtkpacker.h" */
-static const GtkEnumValue _gtk_packer_options_values[] = {
-  { GTK_PACK_EXPAND, "GTK_PACK_EXPAND", "expand" },
-  { GTK_FILL_X, "GTK_FILL_X", "x" },
-  { GTK_FILL_Y, "GTK_FILL_Y", "y" },
-  { 0, NULL, NULL }
-};
-
 /* enumerations from "gtkprivate.h" */
 static const GtkEnumValue _gtk_private_flags_values[] = {
   { PRIVATE_GTK_USER_STYLE, "PRIVATE_GTK_USER_STYLE", "user-style" },
@@ -503,6 +503,7 @@ static const GtkEnumValue _gtk_private_flags_values[] = {
   { PRIVATE_GTK_DIRECTION_SET, "PRIVATE_GTK_DIRECTION_SET", "direction-set" },
   { PRIVATE_GTK_DIRECTION_LTR, "PRIVATE_GTK_DIRECTION_LTR", "direction-ltr" },
   { PRIVATE_GTK_ANCHORED, "PRIVATE_GTK_ANCHORED", "anchored" },
+  { PRIVATE_GTK_CHILD_VISIBLE, "PRIVATE_GTK_CHILD_VISIBLE", "child-visible" },
   { 0, NULL, NULL }
 };
 
@@ -556,6 +557,7 @@ static const GtkEnumValue _gtk_rc_token_type_values[] = {
   { GTK_RC_TOKEN_LOWEST, "GTK_RC_TOKEN_LOWEST", "lowest" },
   { GTK_RC_TOKEN_GTK, "GTK_RC_TOKEN_GTK", "gtk" },
   { GTK_RC_TOKEN_APPLICATION, "GTK_RC_TOKEN_APPLICATION", "application" },
+  { GTK_RC_TOKEN_THEME, "GTK_RC_TOKEN_THEME", "theme" },
   { GTK_RC_TOKEN_RC, "GTK_RC_TOKEN_RC", "rc" },
   { GTK_RC_TOKEN_HIGHEST, "GTK_RC_TOKEN_HIGHEST", "highest" },
   { GTK_RC_TOKEN_ENGINE, "GTK_RC_TOKEN_ENGINE", "engine" },
@@ -636,20 +638,6 @@ static const GtkEnumValue _gtk_tree_model_flags_values[] = {
   { 0, NULL, NULL }
 };
 
-/* enumerations from "gtktreeselection.h" */
-static const GtkEnumValue _gtk_tree_selection_mode_values[] = {
-  { GTK_TREE_SELECTION_SINGLE, "GTK_TREE_SELECTION_SINGLE", "single" },
-  { GTK_TREE_SELECTION_MULTI, "GTK_TREE_SELECTION_MULTI", "multi" },
-  { 0, NULL, NULL }
-};
-
-/* enumerations from "gtktreesortable.h" */
-static const GtkEnumValue _gtk_tree_sort_order_values[] = {
-  { GTK_TREE_SORT_ASCENDING, "GTK_TREE_SORT_ASCENDING", "ascending" },
-  { GTK_TREE_SORT_DESCENDING, "GTK_TREE_SORT_DESCENDING", "descending" },
-  { 0, NULL, NULL }
-};
-
 /* enumerations from "gtktreeview.h" */
 static const GtkEnumValue _gtk_tree_view_drop_position_values[] = {
   { GTK_TREE_VIEW_DROP_BEFORE, "GTK_TREE_VIEW_DROP_BEFORE", "before" },
@@ -661,7 +649,7 @@ static const GtkEnumValue _gtk_tree_view_drop_position_values[] = {
 
 /* enumerations from "gtktreeviewcolumn.h" */
 static const GtkEnumValue _gtk_tree_view_column_sizing_values[] = {
-  { GTK_TREE_VIEW_COLUMN_RESIZEABLE, "GTK_TREE_VIEW_COLUMN_RESIZEABLE", "resizeable" },
+  { GTK_TREE_VIEW_COLUMN_RESIZABLE, "GTK_TREE_VIEW_COLUMN_RESIZABLE", "resizable" },
   { GTK_TREE_VIEW_COLUMN_AUTOSIZE, "GTK_TREE_VIEW_COLUMN_AUTOSIZE", "autosize" },
   { GTK_TREE_VIEW_COLUMN_FIXED, "GTK_TREE_VIEW_COLUMN_FIXED", "fixed" },
   { 0, NULL, NULL }
@@ -689,6 +677,11 @@ static const GtkEnumValue _gtk_widget_flags_values[] = {
   { GTK_DOUBLE_BUFFERED, "GTK_DOUBLE_BUFFERED", "double-buffered" },
   { 0, NULL, NULL }
 };
+static const GtkEnumValue _gtk_widget_help_type_values[] = {
+  { GTK_WIDGET_HELP_TOOLTIP, "GTK_WIDGET_HELP_TOOLTIP", "tooltip" },
+  { GTK_WIDGET_HELP_WHATS_THIS, "GTK_WIDGET_HELP_WHATS_THIS", "whats-this" },
+  { 0, NULL, NULL }
+};
 
 /* Generated data ends here */
 
